@@ -7,7 +7,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2009-2013 Horde LLC (http://www.horde.org)
+ * @copyright 2009-2014 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -20,7 +20,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2009-2013 Horde LLC (http://www.horde.org)
+ * @copyright 2009-2014 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  *
@@ -191,6 +191,7 @@ class Horde_ActiveSync
     const FOLDER_TYPE_USER_NOTE                 =  17;
     const FOLDER_TYPE_UNKNOWN                   =  18;
     const FOLDER_TYPE_RECIPIENT_CACHE           =  19;
+    // @TODO, remove const definition in H6, not used anymore.
     const FOLDER_TYPE_DUMMY                     =  999999;
 
     /* Origin of changes **/
@@ -231,6 +232,7 @@ class Horde_ActiveSync
     const CHANGE_TYPE_FLAGS                     = 'flags';
     const CHANGE_TYPE_MOVE                      = 'move';
     const CHANGE_TYPE_FOLDERSYNC                = 'foldersync';
+    const CHANGE_TYPE_SOFTDELETE                = 'softdelete';
 
     /* Internal flags to indicate change is a change in reply/forward state */
     const CHANGE_REPLY_STATE                    = '@--reply--@';
@@ -283,15 +285,11 @@ class Horde_ActiveSync
     const RESOLVE_RESULT_GAL                    = 1;
     const RESOLVE_RESULT_ADDRESSBOOK            = 2;
 
-    /* Maximum number of times we can see the same synckey before we call it an
-    infinite loop */
-    const MAXIMUM_SYNCKEY_COUNT                 = 10;
-
     /* Auth failure reasons */
     const AUTH_REASON_USER_DENIED               = 'user';
     const AUTH_REASON_DEVICE_DENIED             = 'device';
 
-    const LIBRARY_VERSION                       = '2.5.6';
+    const LIBRARY_VERSION                       = '2.12.3';
 
     /**
      * Logger
@@ -360,7 +358,7 @@ class Horde_ActiveSync
      *
      * @var Horde_ActiveSync_Device
      */
-    protected $_device;
+    static protected $_device;
 
     /**
      * Wbxml encoder
@@ -383,33 +381,6 @@ class Horde_ActiveSync
      */
     protected $_collectionsObj;
 
-    /**
-     * Map of commands when query string is base64 encoded (EAS 12.1)
-     *
-     * @var array
-     */
-    protected $_commandMap = array(
-        0  => 'Sync',
-        1  => 'SendMail',
-        2  => 'SmartForward',
-        3  => 'SmartReply',
-        4  => 'GetAttachment',
-        9  => 'FolderSync',
-        10 => 'FolderCreate',
-        11 => 'FolderDelete',
-        12 => 'FolderUpdate',
-        13 => 'MoveItems',
-        14 => 'GetItemEstimate',
-        15 => 'MeetingResponse',
-        16 => 'Search',
-        17 => 'Settings',
-        18 => 'Ping',
-        19 => 'ItemOperations',
-        20 => 'Provision',
-        21 => 'ResolveRecipients',
-        22 => 'ValidateCert'
-    );
-
     /**
      * Global error flag.
      *
@@ -417,6 +388,13 @@ class Horde_ActiveSync
      */
     protected $_globalError = false;
 
+    /**
+     * Process id (used in logging).
+     *
+     * @var integer
+     */
+    protected $_procid;
+
     /**
      * Supported EAS versions.
      *
@@ -437,6 +415,7 @@ class Horde_ActiveSync
      * @since 2.4.0
      *
      * @return Horde_ActiveSync_Message_Base   The concrete message object.
+     * @todo For H6, move to Horde_ActiveSync_Message_Base::factory()
      */
     static public function messageFactory($message)
     {
@@ -445,7 +424,10 @@ class Horde_ActiveSync
             throw new InvalidArgumentException(sprintf('Class %s does not exist.', $class));
         }
 
-        return new $class(array('logger' => self::$_logger, 'protocolversion' => self::$_version));
+        return new $class(array(
+            'logger' => self::$_logger,
+            'protocolversion' => self::$_version,
+            'device' => self::$_device));
     }
 
     /**
@@ -479,6 +461,8 @@ class Horde_ActiveSync
         // Wbxml handlers
         $this->_encoder = $encoder;
         $this->_decoder = $decoder;
+
+        $this->_procid = getmypid();
     }
 
     /**
@@ -506,8 +490,8 @@ class Horde_ActiveSync
     {
         return new Horde_ActiveSync_SyncCache(
             $this->_state,
-            $this->_device->id,
-            $this->_device->user,
+            self::$_device->id,
+            self::$_device->user,
             self::$_logger
         );
     }
@@ -540,15 +524,25 @@ class Horde_ActiveSync
         if (!empty($serverVars['PHP_AUTH_PW'])) {
             $user = empty($username) ? $serverVars['PHP_AUTH_USER'] : $username;
             $pass = $serverVars['PHP_AUTH_PW'];
-        } elseif (!empty($serverVars['Authorization'])) {
-            $hash = base64_decode(str_replace('Basic ', '', $serverVars['Authorization']));
+        } elseif (!empty($serverVars['HTTP_AUTHORIZATION']) || !empty($serverVars['Authorization'])) {
+            // Some clients use the non-standard 'Authorization' header.
+            $authorization = !empty($serverVars['HTTP_AUTHORIZATION'])
+                ? $serverVars['HTTP_AUTHORIZATION']
+                : $serverVars['Authorization'];
+            $hash = base64_decode(str_replace('Basic ', '', $authorization));
             if (strpos($hash, ':') !== false) {
                 list($user, $pass) = explode(':', $hash, 2);
             }
             $user = !empty($username) ? $username : $user;
+        } elseif (!empty($username)) {
+            // Might not need a password, could be using X509 cert.
+            $user = $username;
         } else {
             // No provided username or Authorization header.
-            self::$_logger->debug('Client did not provide authentication data.');
+            self::$_logger->notice(sprintf(
+                '[%s] Client did not provide authentication data.',
+                $this->_procid)
+            );
             return false;
         }
         $user = $this->_driver->getUsernameFromEmail($user);
@@ -573,25 +567,6 @@ class Horde_ActiveSync
             return false;
         }
 
-        // Some devices (incorrectly) only send the username in the httpauth
-        $get = $this->getGetVars();
-        if ($this->_request->getMethod() == 'POST' && empty($get['User'])) {
-            if ($serverVars['PHP_AUTH_USER']) {
-                $get['User'] = $serverVars['PHP_AUTH_USER'];
-            } elseif ($serverVars['HTTP_AUTHORIZATION']) {
-                $hash = str_replace('Basic ', '', $serverVars['Authorization']);
-                $hash = base64_decode($hash);
-                if (strpos($hash, ':') !== false) {
-                    list($get['User'], $pass) = explode(':', $hash, 2);
-                }
-            }
-
-            if (empty($get['User'])) {
-                self::$_logger->err('Missing required parameters.');
-                throw new Horde_ActiveSync_Exception('Your device requested the ActiveSync URL wihtout required parameters.');
-            }
-        }
-
         if (!$this->_driver->setup($user)) {
             return false;
         }
@@ -637,11 +612,12 @@ class Horde_ActiveSync
         case 'provisioning':
         case 'multipart':
         case 'certPath':
-        case 'device':
             $property = '_' . $property;
             return $this->$property;
         case 'logger':
             return self::$_logger;
+        case 'device':
+            return self::$_device;
         default:
             throw new InvalidArgumentException(sprintf(
                 'The property %s does not exist',
@@ -664,11 +640,13 @@ class Horde_ActiveSync
 
     protected function _setLogger(array $options)
     {
-        self::$_logger = $this->_loggerFactory->create($options);
-        $this->_encoder->setLogger(self::$_logger);
-        $this->_decoder->setLogger(self::$_logger);
-        $this->_driver->setLogger(self::$_logger);
-        $this->_state->setLogger(self::$_logger);
+        if (!empty($this->_loggerFactory)) {
+            self::$_logger = $this->_loggerFactory->create($options);
+            $this->_encoder->setLogger(self::$_logger);
+            $this->_decoder->setLogger(self::$_logger);
+            $this->_driver->setLogger(self::$_logger);
+            $this->_state->setLogger(self::$_logger);
+        }
     }
 
     /**
@@ -729,14 +707,17 @@ class Horde_ActiveSync
         // Autodiscovery handles authentication on it's own.
         if ($cmd == 'Autodiscover') {
             $request = new Horde_ActiveSync_Request_Autodiscover($this, new Horde_ActiveSync_Device($this->_state));
-            $request->setLogger(self::$_logger);
 
-            $result = $request->handle();
+            if (!empty(self::$_logger)) {
+                $request->setLogger(self::$_logger);
+            }
+
+            $result = $request->handle($this->_request);
             $this->_driver->clearAuthentication();
             return $result;
         }
 
-        if (!$this->authenticate()) {
+        if (!$this->authenticate(!empty($get['User']) ? $get['User'] : '')) {
             $this->activeSyncHeader();
             $this->versionHeader();
             $this->commandsHeader();
@@ -746,9 +727,9 @@ class Horde_ActiveSync
         // Set provisioning support now that we are authenticated.
         $this->setProvisioning($this->_driver->getProvisioning());
 
-        self::$_logger->debug(sprintf(
+        self::$_logger->info(sprintf(
             '[%s] %s request received for user %s',
-            getmypid(),
+            $this->_procid,
             strtoupper($cmd),
             $this->_driver->getUser())
         );
@@ -776,62 +757,96 @@ class Horde_ActiveSync
         if (!$this->_state->deviceExists($devId, $this->_driver->getUser())) {
             // Device might exist, but with a new (additional) user account
             if ($this->_state->deviceExists($devId)) {
-                $device = $this->_state->loadDeviceInfo($devId);
+                self::$_device = $this->_state->loadDeviceInfo($devId);
             } else {
-                $device = new Horde_ActiveSync_Device($this->_state);
+                self::$_device = new Horde_ActiveSync_Device($this->_state);
             }
-            $device->policykey = 0;
-            $device->userAgent = $this->_request->getHeader('User-Agent');
-            $device->deviceType = !empty($get['DeviceType']) ? $get['DeviceType'] : '';
-            $device->rwstatus = self::RWSTATUS_NA;
-            $device->user = $this->_driver->getUser();
-            $device->id = $devId;
-            $device->properties['version'] = $version;
-            // @TODO: Remove is_callable check (and extra else clause) for H6.
+            self::$_device->policykey = 0;
+            self::$_device->userAgent = $this->_request->getHeader('User-Agent');
+            self::$_device->deviceType = !empty($get['DeviceType']) ? $get['DeviceType'] : '';
+            self::$_device->rwstatus = self::RWSTATUS_NA;
+            self::$_device->user = $this->_driver->getUser();
+            self::$_device->id = $devId;
+            self::$_device->needsVersionUpdate($this->getSupportedVersions());
+            self::$_device->version = $version;
+            // @TODO: Remove is_callable check for H6.
+            //        Combine this with the modifyDevice callback? Allow $device
+            //        to be modified here?
             if (is_callable(array($this->_driver, 'createDeviceCallback'))) {
-                $callback_ret = $this->_driver->createDeviceCallback($device);
+                $callback_ret = $this->_driver->createDeviceCallback(self::$_device);
                 if ($callback_ret !== true) {
                     $msg = sprintf(
                         'The device %s was disallowed for user %s per policy settings.',
-                        $device->id,
-                        $device->user);
-                    $this->_logger->err($msg);
+                        self::$_device->id,
+                        self::$_device->user);
+                    self::$_logger->err($msg);
                     if ($version > self::VERSION_TWELVEONE) {
                         $this->_globalError = $callback_ret;
                     } else {
                         throw new Horde_ActiveSync_Exception($msg);
                     }
                 } else {
-                    $device->save();
+                    // Give the driver a chance to modify device properties.
+                    if (is_callable(array($this->_driver, 'modifyDeviceCallback'))) {
+                        self::$_device = $this->_driver->modifyDeviceCallback(self::$_device);
+                    }
                 }
-            } else {
-                $device->save();
             }
         } else {
-            $device = $this->_state->loadDeviceInfo($devId, $this->_driver->getUser());
-            $device->properties['version'] = $version;
-            $device->save();
-            if (is_callable(array($this->_driver, 'deviceCallback'))) {
-                $callback_ret = $this->_driver->deviceCallback($device);
-                if ($callback_ret !== true) {
-                    $msg = sprintf(
-                        'The device %s was disallowed for user %s per policy settings.',
-                        $device->id,
-                        $device->user);
-                    $this->_logger->err($msg);
-                    if ($version > self::VERSION_TWELVEONE) {
-                        $this->_globalError = $callback_ret;
-                    } else {
-                        throw new Horde_ActiveSync_Exception($msg);
-                    }
+            self::$_device = $this->_state->loadDeviceInfo($devId, $this->_driver->getUser());
+
+            // If the device state was removed from storage, we may lose the
+            // device properties, so try to repopulate what we can. userAgent
+            // is ALWAYS available, so if it's missing, the state is gone.
+            if (empty(self::$_device->userAgent)) {
+                self::$_device->userAgent = $this->_request->getHeader('User-Agent');
+                self::$_device->deviceType = !empty($get['DeviceType']) ? $get['DeviceType'] : '';
+                self::$_device->user = $this->_driver->getUser();
+            }
+
+            if (empty(self::$_device->version)) {
+                self::$_device->version = $version;
+            }
+            if (self::$_device->version < $this->_maxVersion &&
+                self::$_device->needsVersionUpdate($this->getSupportedVersions())) {
+                $needMsRp = true;
+            }
+
+            // Give the driver a chance to modify device properties.
+            if (is_callable(array($this->_driver, 'modifyDeviceCallback'))) {
+                self::$_device = $this->_driver->modifyDeviceCallback(self::$_device);
+            }
+        }
+
+        self::$_device->save();
+        if (is_callable(array($this->_driver, 'deviceCallback'))) {
+            $callback_ret = $this->_driver->deviceCallback(self::$_device);
+            if ($callback_ret !== true) {
+                $msg = sprintf(
+                    'The device %s was disallowed for user %s per policy settings.',
+                    self::$_device->id,
+                    self::$_device->user);
+                self::$_logger->err($msg);
+                if ($version > self::VERSION_TWELVEONE) {
+                    $this->_globalError = $callback_ret;
+                } else {
+                    throw new Horde_ActiveSync_Exception($msg);
                 }
             }
         }
 
-        // Always set the version information instead of caching it, some
-        // clients may allow changing the protocol version.
-        $device->version = $version;
-        $this->_device = $device;
+        // Lastly, check if the device has been set to blocked.
+        if (self::$_device->blocked) {
+            $msg = sprintf(
+                'The device %s was blocked.',
+                self::$_device->id);
+            self::$_logger->err($msg);
+            if ($version > self::VERSION_TWELVEONE) {
+                $this->_globalError = Horde_ActiveSync_Status::DEVICE_BLOCKED_FOR_USER;
+            } else {
+                throw new Horde_ActiveSync_Exception($msg);
+            }
+        }
 
         // Don't bother with everything else if all we want are Options
         if ($cmd == 'Options') {
@@ -846,30 +861,45 @@ class Horde_ActiveSync
         // Support Multipart response for ITEMOPERATIONS requests?
         $headers = $this->_request->getHeaders();
         if ((!empty($headers['ms-asacceptmultipart']) && $headers['ms-asacceptmultipart'] == 'T') ||
-            (isset($get['Options']) && ($get['Options'] & 0x02))) {
+            !empty($get['AcceptMultiPart'])) {
             $this->_multipart = true;
-            self::$_logger->debug('MULTIPART REQUEST');
+            self::$_logger->info(sprintf(
+                '[%s] Requesting multipart data.',
+                $this->_procid)
+            );
         }
 
         // Load the request handler to handle the request
         // We must send the eas header here, since some requests may start
         // output and be large enough to flush the buffer (e.g., GetAttachment)
+        // See Bug: 12486
         $this->activeSyncHeader();
         if ($cmd != 'GetAttachment') {
             $this->contentTypeHeader();
         }
 
         // Should we announce a new version is available to the client?
-        if ($device->version < $this->_maxVersion) {
+        if (!empty($needMsRp)) {
+            self::$_logger->info(sprintf(
+                '[%s] Announcing X-MS-RP to client.',
+                $this->_procid)
+            );
             header("X-MS-RP: ". $this->getSupportedVersions());
         }
 
+        // @TODO: Look at getting rid of having to set the version in the driver
+        //        and get it from the device object for H6.
+        $this->_driver->setDevice(self::$_device);
         $class = 'Horde_ActiveSync_Request_' . basename($cmd);
         if (class_exists($class)) {
             $request = new $class($this);
             $request->setLogger(self::$_logger);
             $result = $request->handle();
-            $this->_driver->clearAuthentication();
+            self::$_logger->info(sprintf(
+                '[%s] Maximum memory usage for ActiveSync request: %d bytes.',
+                $this->_procid,
+                memory_get_peak_usage())
+            );
 
             return $result;
         }
@@ -971,9 +1001,9 @@ class Horde_ActiveSync
      */
     public function getPolicyKey()
     {
+        // Policy key can come from header or encoded request parameters.
         $this->_policykey = $this->_request->getHeader('X-MS-PolicyKey');
         if (empty($this->_policykey)) {
-            // Try the get request.
             $get = $this->getGetVars();
             if (!empty($get['PolicyKey'])) {
                 $this->_policykey = $get['PolicyKey'];
@@ -999,12 +1029,6 @@ class Horde_ActiveSync
         if (empty(self::$_version)) {
             $get = $this->getGetVars();
             self::$_version = empty($get['ProtVer']) ? '1.0' : $get['ProtVer'];
-            if (self::$_version == 121) {
-                self::$_version = 12.1;
-            }
-            if (self::$_version == 141) {
-                self::$_version = 14.1;
-            }
         }
         return self::$_version;
     }
@@ -1023,13 +1047,14 @@ class Horde_ActiveSync
 
         $results = array();
         $get = $this->_request->getGetVars();
+
+        // Do we need to decode the request parameters?
         if (!isset($get['Cmd']) && !isset($get['DeviceId']) && !isset($get['DeviceType'])) {
             $serverVars = $this->_request->getServerVars();
             if (isset($serverVars['QUERY_STRING']) && strlen($serverVars['QUERY_STRING']) >= 10) {
-                $decoded = Horde_ActiveSync_Utils::decodeBase64($serverVars['QUERY_STRING']);
-                $results['DeviceId'] = $decoded['DevID'];
-                $results['PolicyKey'] = $decoded['PolKey'];
-                switch ($decoded['DevType']) {
+                $results = Horde_ActiveSync_Utils::decodeBase64($serverVars['QUERY_STRING']);
+                // Normalize values.
+                switch ($results['DeviceType']) {
                 case 'PPC':
                     $results['DeviceType'] = 'PocketPC';
                     break;
@@ -1037,40 +1062,13 @@ class Horde_ActiveSync
                     $results['DeviceType'] = 'SmartPhone';
                     break;
                 case 'WP':
+                case 'WP8':
                     $results['DeviceType'] = 'WindowsPhone';
+                    break;
+                case 'android':
+                case 'android40':
+                    $results['DeviceType'] = 'android';
                 }
-                $results['Cmd'] = $this->_commandMap[$decoded['Command']];
-                if (isset($decoded['AttachmentName'])) {
-                    $results['AttachmentName'] = $decoded['AttachmentName'];
-                }
-                if (isset($decoded['ItemId'])) {
-                    $results['ItemId'] = $decoded['ItemId'];
-                }
-                if (isset($decoded['CollectionId'])) {
-                    $results['CollectionId'] = $decoded['CollectionId'];
-                }
-                if (isset($decoded['CollectionName'])) {
-                    $results['CollectionName'] = $decoded['CollectionName'];
-                }
-                if (isset($decoded['ParentId'])) {
-                    $results['ParentId'] = $decoded['ParentId'];
-                }
-                if (isset($decoded['LongId'])) {
-                    $results['LongId'] = $decoded['LongId'];
-                }
-                if (isset($decoded['Occurrence'])) {
-                    $results['Occurrence'] = $decoded['Occurrence'];
-                }
-                if (isset($decoded['Options'])) {
-                    $results['Options'] = bin2hex($decoded['Options']) * 1;
-                }
-                if (isset($decoded['User'])) {
-                    $results['User'] = $decoded['User'];
-                }
-                if (isset($decoded['ProtVer'])) {
-                    $results['ProtVer'] = $decoded['ProtVer'];
-                }
-
                 $this->_get = $results;
             }
         } else {
