@@ -7,7 +7,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2013 Horde LLC (http://www.horde.org)
+ * @copyright 2013-2014 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -20,7 +20,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2010-2013 Horde LLC (http://www.horde.org)
+ * @copyright 2010-2014 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -30,6 +30,7 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
     const COLLECTION_ERR_SERVER              = -2;
     const COLLECTION_ERR_STALE               = -3;
     const COLLECTION_ERR_SYNC_REQUIRED       = -4;
+    const COLLECTION_ERR_PING_NEED_FULL      = -5;
 
     /**
      * The collection data
@@ -160,7 +161,12 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
                 $collection['class'] = $this->getCollectionClass($collection['id']);
             }
             if (empty($collection['serverid'])) {
-                $collection['serverid'] = $this->getBackendIdForFolderUid($collection['id']);
+                try {
+                    $collection['serverid'] = $this->getBackendIdForFolderUid($collection['id']);
+                } catch (Horde_ActiveSync_Exception $e) {
+                    $this->_logger->err($e->getMessage());
+                    continue;
+                }
             }
             $this->_collections[$collection['id']] = $collection;
             $this->_logger->info(sprintf(
@@ -261,13 +267,20 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
      */
     public function addCollection(array $collection, $requireSyncKey = false)
     {
-
         if ($requireSyncKey && empty($collection['synckey'])) {
             $cached_collections = $this->_cache->getCollections(false);
             $collection['synckey'] = !empty($cached_collections[$collection['id']])
                 ? $cached_collections[$collection['id']]['lastsynckey']
                 : 0;
 
+            if ($collection['synckey'] === 0) {
+                $this->_logger->err(sprintf('[%s] Attempting to add a collection
+                    to the sync cache while requiring a synckey, but no
+                    synckey could be found. Most likely a client error in
+                    requesting a collection during PING before it has issued a
+                    SYNC.', $this->_procid));
+            }
+
             $this->_logger->info(sprintf(
                 '[%s] Obtained synckey for collection %s from cache: %s',
                 $this->_procid, $collection['id'], $collection['synckey']));
@@ -294,11 +307,20 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
      * @param $id  The uid.
      *
      * @return string  The backend server id.
+     * @throws Horde_ActiveSync_Exception
      */
     public function getBackendIdForFolderUid($folderid)
     {
+        // Always use RI for recipient cache.
+        if ($folderid == 'RI') {
+            return $folderid;
+        }
         $folder = $this->_cache->getFolder($folderid);
-        return $folder['serverid'];
+        if ($folder) {
+            return $folder['serverid'];
+        } else {
+            throw new Horde_ActiveSync_Exception('Folder not found in cache.');
+        }
     }
 
     /**
@@ -308,10 +330,18 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
      *
      * @return string The EAS uid.
      */
-
-    public function getFolderUidForBackendId($id)
+    public function getFolderUidForBackendId($folderid)
     {
-        return $this->_as->state->getFolderUidForBackendId($id);
+        // Always use 'RI' for Recipient cache.
+        if ($folderid == 'RI') {
+            return $folderid;
+        }
+        $map = $this->_as->state->getFolderUidToBackendIdMap();
+        if (empty($map[$folderid])) {
+            return false;
+        }
+
+        return $map[$folderid];
     }
 
     /**
@@ -421,6 +451,9 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
      */
     public function getCollectionClass($id)
     {
+        if ($id == 'RI') {
+            return $id;
+        }
         if (isset($this->_cache->folders[$id]['class'])) {
             $class = $this->_cache->folders[$id]['class'];
             $this->_logger->info(sprintf(
@@ -497,13 +530,15 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
     }
 
     /**
-     * Return whether or not we want a looping sync.
+     * Return whether or not we want a looping sync. We can do a looping sync
+     * if we have no imported changes AND we have either a hbinterval, wait,
+     * or a shortSync.
      *
      * @return boolean  True if we want a looping sync, false otherwise.
      */
     public function canDoLoopingSync()
     {
-        return ($this->_shortSyncRequest || $this->_cache->hbinterval !== false || $this->_cache->wait !== false);
+        return !$this->_importedChanges && ($this->_shortSyncRequest || $this->_cache->hbinterval !== false || $this->_cache->wait !== false);
     }
 
     /**
@@ -735,8 +770,9 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
         $csk = $this->_cache->confirmed_synckeys;
         if ($csk) {
             $this->_logger->info(sprintf(
-                'Confirmed Synckeys contains %s',
-                print_r($csk, true))
+                '[%s] Confirmed Synckeys contains %s',
+                $this->_procid,
+                serialize($csk))
             );
             $this->_logger->err('Some synckeys were not confirmed. Requesting full SYNC');
             $this->save();
@@ -808,57 +844,6 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
         }
     }
 
-    /**
-     * Check the loop counters for any possible infinite sync attempts. Will
-     * reset the collection state for any collection that has reached the
-     * MAXIMUM_SYNCKEY_COUNTER value.
-     *
-     * @return boolean  True if counters validate, false if we have reached the
-     *                  MAXIMUM_SYNCKEY_COUNTER value and cleared the state.
-     */
-    public function checkLoopCounters()
-    {
-        $counters = $this->_cache->synckeycounter;
-        foreach ($this->_collections as $id => $collection) {
-            if (!empty($counters[$id][$collection['synckey']]) &&
-                $counters[$id][$collection['synckey']] > Horde_ActiveSync::MAXIMUM_SYNCKEY_COUNT) {
-
-                $this->_logger->err(sprintf(
-                    '[%s] Reached MAXIMUM_SYNCKEY_COUNT for synckey %s, collection %s, possible sync loop, so sending SERVER_ERROR.',
-                    $this->_procid,
-                    $collection['synckey'],
-                    $id));
-                return false;
-            } elseif (empty($counters[$collection['id']][$collection['synckey']])) {
-                // First time for this synckey. Remove others.
-                $counters[$collection['id']] = array($collection['synckey'] => 0);
-            } else {
-                $this->_logger->warn('LOOP COUNTER: ' . $collection['synckey'] . ' : ' . $counters[$id][$collection['synckey']]);
-            }
-        }
-        $this->_cache->synckeycounter = $counters;
-
-        return true;
-    }
-
-    /**
-     * Increment the loop detection counter.
-     *
-     * @param string $id   The collection id.
-     * @param string $key  The synckey.
-     */
-    public function incrementLoopCounter($id, $key)
-    {
-        $counters = $this->_cache->synckeycounter;
-        if (empty($counters[$id][$key])) {
-            $counters[$id][$key] = 0;
-        }
-        if (++$counters[$id][$key] > 1) {
-            $this->_logger->warn('Incrementing loop counter. We saw this synckey before.');
-        }
-        $this->_cache->synckeycounter = $counters;
-    }
-
     /**
      * Check for an update FILTERTYPE
      *
@@ -1026,21 +1011,32 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
                         $this->_procid,
                         $id)
                     );
+                    if (!empty($options['pingable'])) {
+                        return self::COLLECTION_ERR_PING_NEED_FULL;
+                    }
                     $dataavailable = true;
                     $this->setGetChangesFlag($id);
                     continue;
                 } catch (Horde_ActiveSync_Exception_InvalidRequest $e) {
+                    // Thrown when state is unable to be initialized because the
+                    // collection has not yet been synched, but was requested to
+                    // be pinged.
                     $this->_logger->err(sprintf(
-                        '[%s] Heartbeat terminating: %s',
+                        '[%s] Unable to initialize state for %s. Ignoring during pollForChanges: %s.',
                         $this->_procid,
+                        $id,
                         $e->getMessage()));
-                    $this->setGetChangesFlag($id);
-                    $dataavailable = true;
+                    continue;
                 } catch (Horde_ActiveSync_Exception_FolderGone $e) {
                     $this->_logger->warn('Folder gone for collection ' . $collection['id']);
                     return self::COLLECTION_ERR_FOLDERSYNC_REQUIRED;
                 } catch (Horde_ActiveSync_Exception $e) {
                     $this->_logger->err('Error loading state: ' . $e->getMessage());
+                    $this->_as->state->loadState(
+                        array(),
+                        null,
+                        Horde_ActiveSync::REQUEST_TYPE_SYNC,
+                        $id);
                     $this->setGetChangesFlag($id);
                     $dataavailable = true;
                     continue;
@@ -1172,18 +1168,48 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
      * we are not PINGing.
      *
      * @param boolean $ping  True if this is a PING request, false otherwise.
+     * @param array $ensure  An array of UIDs that should be sent in the
+     *                       current response if possible, and not put off
+     *                       because of a MOREAVAILABLE situation.
      *
      * @return array  The changes array.
      */
-    public function getCollectionChanges($ping = false)
+    public function getCollectionChanges($ping = false, array $ensure = array())
     {
         if (empty($this->_changes)) {
             $this->_changes = $this->_as->state->getChanges(array('ping' => $ping));
         }
 
+        if (!empty($ensure)) {
+            $this->_changes = $this->_reorderChanges($ensure);
+        }
+
         return $this->_changes;
     }
 
+    protected function _reorderChanges(array $ensure)
+    {
+        $changes = array();
+        foreach ($this->_changes as $change) {
+            if (array_search($change['id'], $ensure) !== false) {
+                $this->_logger->info(sprintf(
+                    'Placing %s at beginning of changes array.', $change['id']));
+                array_unshift($changes, $change);
+            } else {
+                $changes[] = $change;
+            }
+        }
+
+        return $changes;
+    }
+
+    /**
+     * Return the count of the current collection's chagnes.
+     *
+     * @param boolean $ping  Only ping the collection if true.
+     *
+     * @return integer  The change count.
+     */
     public function getCollectionChangeCount($ping = false)
     {
         if (empty($this->_changes)) {
@@ -1201,4 +1227,4 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
         return new ArrayIterator($this->_collections);
     }
 
-}
\ No newline at end of file
+}
