@@ -18,7 +18,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2009-2013 Horde LLC (http://www.horde.org)
+ * @copyright 2009-2014 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -30,7 +30,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2009-2013 Horde LLC (http://www.horde.org)
+ * @copyright 2009-2014 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -101,6 +101,13 @@ class Horde_ActiveSync_Connector_Exporter
      */
     protected $_as;
 
+    /**
+     * Process id for logging.
+     *
+     * @var integer
+     */
+    protected $_procid;
+
     /**
      * Const'r
      *
@@ -116,6 +123,7 @@ class Horde_ActiveSync_Connector_Exporter
         $this->_as = $as;
         $this->_encoder = $encoder;
         $this->_logger = $as->logger;
+        $this->_procid = getmypid();
     }
 
     /**
@@ -136,7 +144,9 @@ class Horde_ActiveSync_Connector_Exporter
     /**
      * Sends the next change in the set to the client.
      *
-     * @return boolean  True if more changes can be sent, otherwise false.
+     * @return boolean|Horde_Exception True if more changes can be sent false if
+     *                                 all changes were sent, Horde_Exception if
+     *                                 there was an error sending an item.
      */
     public function sendNextChange()
     {
@@ -156,12 +166,13 @@ class Horde_ActiveSync_Connector_Exporter
                             $change['id'],
                             $folder->parentid,
                             $folder->displayname,
-                            $folder->_serverid);
+                            $folder->_serverid,
+                            $folder->type);
                         $this->folderChange($folder);
                     } else {
                         $this->_logger->err(sprintf(
                             '[%s] Error stating %s: ignoring.',
-                            getmypid(), $change['id']));
+                            $this->_procid, $change['id']));
                         $stat = array('id' => $change['id'], 'mod' => $change['id'], 0);
                     }
                     // Update the state.
@@ -183,12 +194,17 @@ class Horde_ActiveSync_Connector_Exporter
         } else {
             if ($this->_step < count($this->_changes)) {
                 $change = $this->_changes[$this->_step];
+
+                // Ignore this change, no UID value, keep trying until we get a
+                // good entry or we run out of entries.
                 while (empty($change['id']) && $this->_step < count($this->_changes) - 1) {
                     $this->_logger->err('Missing UID value for an entry in: ' . $this->_currentCollection['id']);
                     $this->_step++;
                     $change = $this->_changes[$this->_step];
                 }
 
+                // Actually export the change by calling the appropriate
+                // method to output the correct wbxml for this change.
                 if (empty($change['ignore'])) {
                     switch($change['type']) {
                     case Horde_ActiveSync::CHANGE_TYPE_CHANGE:
@@ -200,9 +216,17 @@ class Horde_ActiveSync_Connector_Exporter
                             $message->flags = (isset($change['flags'])) ? $change['flags'] : 0;
                             $this->messageChange($change['id'], $message);
                         } catch (Horde_Exception_NotFound $e) {
-                            $this->_logger->err('Message gone or error reading message from server: ' . $e->getMessage());
+                            $this->_logger->err(sprintf(
+                                '[%s] Message gone or error reading message from server: %s',
+                                $this->_procid, $e->getMessage()));
+                            $this->_as->state->updateState($change['type'], $change);
+                            $this->_step++;
+                            return $e;
                         } catch (Horde_ActiveSync_Exception $e) {
-                            $this->_logger->err('Unknown backend error skipping message: ' . $e->getMessage());
+                            $this->_logger->err(sprintf(
+                                '[%s] Unknown backend error skipping message: %s',
+                                $this->_procid,
+                                $e->getMessage()));
                         }
                         break;
 
@@ -210,6 +234,10 @@ class Horde_ActiveSync_Connector_Exporter
                         $this->messageDeletion($change['id']);
                         break;
 
+                    case Horde_ActiveSync::CHANGE_TYPE_SOFTDELETE:
+                        $this->messageDeletion($change['id'], true);
+                        break;
+
                     case Horde_ActiveSync::CHANGE_TYPE_FLAGS:
                         // Read flag.
                         $message = Horde_ActiveSync::messageFactory('Mail');
@@ -272,9 +300,11 @@ class Horde_ActiveSync_Connector_Exporter
         if ($message->getClass() != $this->_currentCollection['class'] ||
             in_array($id, $this->_seenObjects)) {
             $this->_logger->err(sprintf(
-                '[%s] IGNORING message %s since it looks like it was already sent or does not belong to this collection.',
-                getmypid(),
-                $id));
+                '[%s] IGNORING message %s since it looks like it was already sent or does not belong to this collection. Class: %s, CurrentClass: %s',
+                $this->_procid,
+                $id,
+                $message->getClass(),
+                $this->_currentCollection['class']));
             return;
         }
 
@@ -302,10 +332,15 @@ class Horde_ActiveSync_Connector_Exporter
      * Stream a message deletion to the PIM
      *
      * @param string $id  The uid of the message we are deleting.
+     * @param boolean $soft  If true, send a SOFTDELETE, otherwise a REMOVE.
      */
-    public function messageDeletion($id)
+    public function messageDeletion($id, $soft = false)
     {
-        $this->_encoder->startTag(Horde_ActiveSync::SYNC_REMOVE);
+        if ($soft) {
+            $this->_encoder->startTag(Horde_ActiveSync::SYNC_SOFTDELETE);
+        } else {
+            $this->_encoder->startTag(Horde_ActiveSync::SYNC_REMOVE);
+        }
         $this->_encoder->startTag(Horde_ActiveSync::SYNC_SERVERENTRYID);
         $this->_encoder->content($id);
         $this->_encoder->endTag();
