@@ -7,7 +7,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2010-2013 Horde LLC (http://www.horde.org)
+ * @copyright 2010-2014 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -22,7 +22,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2010-2013 Horde LLC (http://www.horde.org)
+ * @copyright 2010-2014 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -68,7 +68,7 @@ abstract class Horde_ActiveSync_Driver_Base
      *
      * @var float
      */
-    protected $_version;
+    protected $_version = Horde_ActiveSync::VERSION_FOURTEENONE;
 
     /**
      * The state driver for this request. Needs to be injected into this class.
@@ -77,8 +77,30 @@ abstract class Horde_ActiveSync_Driver_Base
      */
     protected $_state;
 
+    /**
+     * The device object
+     *
+     * @var Horde_ActiveSync_Device
+     * @since  2.12.0
+     */
+    protected $_device;
+
+    /**
+     * Temporary serverid to uid map. Used when creating the hierarchy
+     * for the first time
+     *
+     * @var array
+     */
     protected $_tempMap = array();
 
+    protected $_typeMap = array(
+        'F' => Horde_ActiveSync::CLASS_EMAIL,
+        'C' => Horde_ActiveSync::CLASS_CONTACTS,
+        'A' => Horde_ActiveSync::CLASS_CALENDAR,
+        'T' => Horde_ActiveSync::CLASS_TASKS,
+        'N' => Horde_ActiveSync::CLASS_NOTES
+    );
+
     /**
      * Const'r
      *
@@ -261,6 +283,16 @@ abstract class Horde_ActiveSync_Driver_Base
         return $bodyprefs;
     }
 
+    /**
+     * Set the currently connected device
+     *
+     * @param Horde_ActiveSync_Device $device  The device object.
+     */
+    public function setDevice(Horde_ActiveSync_Device $device)
+    {
+        $this->_device = $device;
+    }
+
     /**
      * Build a EAS style FB string. Essentially, each digit represents 1/2 hour.
      * The values are as follows:
@@ -344,25 +376,89 @@ abstract class Horde_ActiveSync_Driver_Base
      * serverid before, return the previously created uid, otherwise return
      * a new one.
      *
-     * @param string $imap  The imap server's folder name E.g., INBOX
+     * @param string $id    The server's folder name E.g., INBOX
+     * @param string $type  The folder type, a Horde_ActiveSync::FOLDER_TYPE_*
+     *                      constant. If empty, assumes FOLDER_TYPE_USER_MAIL
      *
-     * @return string  A UUID to be used by activesync to represent the folder.
+     * @return string  A unique identifier for the specified backend folder id.
+     *                 The first character indicates the foldertype as such:
+     *                 'F' - Email
+     *                 'C' - Contact
+     *                 'A' - Appointment
+     *                 'T' - Task
+     *                 'N' - Note
      * @since 2.4.0
      */
-    protected function _getFolderUidForBackendId($imap)
+    protected function _getFolderUidForBackendId($id, $type = null)
     {
-        if ($id = $this->_state->getFolderUidForBackendId($imap)) {
-            return $id;
-        } elseif (empty($this->_tempMap[$imap])) {
-            $this->_tempMap[$imap] = sprintf('%04x%04x', mt_rand( 0, 0xffff ), mt_rand( 0, 0xffff ));
-            $this->_logger->info(sprintf(
-                '[%s] Creating new folder uuid for %s: %s',
-                getmypid(),
-                $imap,
-                $this->_tempMap[$imap]));
+        // Always use 'RI' for Recipient cache.
+        if ($id == 'RI') {
+            return 'RI';
+        }
+        $map = $this->_state->getFolderUidToBackendIdMap();
+        if (!empty($map[$id])) {
+            return $map[$id];
+        } elseif (!empty($this->_tempMap[$id])) {
+            return $this->_tempMap[$id];
         }
 
-        return $this->_tempMap[$imap];
+        // Convert TYPE to CLASS
+        $type = $this->_getClassFromType($type);
+        $rMap = array_flip($this->_typeMap);
+        $prefix = $rMap[$type];
+
+        // None found, generate a new UID.
+        $this->_tempMap[$id] = sprintf('%s%04x%04x', $prefix, mt_rand(0, 0xffff), mt_rand(0, 0xffff));
+        $this->_logger->info(sprintf(
+            '[%s] Creating new folder uuid for %s: %s',
+            getmypid(),
+            $id,
+            $this->_tempMap[$id]));
+
+        return $this->_tempMap[$id];
+    }
+
+    /**
+     * Convert a TYPE constant into it's associated CLASS constant.
+     *
+     * @param integer $type  The TYPE.
+     *
+     * @return string  The CLASS
+     */
+    protected function _getClassFromType($type)
+    {
+        // @todo This is for BC. Assume we are asking for an email collection
+        // if we didn't pass a type. Remove in H6.
+        if (empty($type)) {
+            return Horde_ActiveSync::CLASS_EMAIL;
+        }
+
+        switch ($type) {
+        case Horde_ActiveSync::FOLDER_TYPE_APPOINTMENT:
+        case Horde_ActiveSync::FOLDER_TYPE_USER_APPOINTMENT:
+            return Horde_ActiveSync::CLASS_CALENDAR;
+
+        case Horde_ActiveSync::FOLDER_TYPE_CONTACT:
+        case Horde_ActiveSync::FOLDER_TYPE_USER_CONTACT:
+            return Horde_ActiveSync::CLASS_CONTACTS;
+
+        case Horde_ActiveSync::FOLDER_TYPE_TASK:
+        case Horde_ActiveSync::FOLDER_TYPE_USER_TASK:
+            return Horde_ActiveSync::CLASS_TASKS;
+
+        case Horde_ActiveSync::FOLDER_TYPE_NOTE:
+        case Horde_ActiveSync::FOLDER_TYPE_USER_NOTE:
+            return Horde_ActiveSync::CLASS_NOTES;
+
+        case Horde_ActiveSync::FOLDER_TYPE_INBOX:
+        case Horde_ActiveSync::FOLDER_TYPE_DRAFTS:
+        case Horde_ActiveSync::FOLDER_TYPE_WASTEBASKET:
+        case Horde_ActiveSync::FOLDER_TYPE_SENTMAIL:
+        case Horde_ActiveSync::FOLDER_TYPE_OUTBOX:
+        case Horde_ActiveSync::FOLDER_TYPE_USER_MAIL:
+            return Horde_ActiveSync::CLASS_EMAIL;
+
+        }
     }
 
     /**
@@ -394,8 +490,15 @@ abstract class Horde_ActiveSync_Driver_Base
      * @param string $id           The server's folder id
      * @param string $displayname  The new display name.
      * @param string $parent       The folder's parent, if needed.
+     * @param string $uid          The existing folder uid, if this is an edit.
+     *                             @since 2.9.0 (@todo Look at this for H6. It's
+     *                             here now to save an extra DB lookup for data
+     *                             we already have.)
+     *
+     * @return Horde_ActiveSync_Message_Folder
+     * @throws  Horde_ActiveSync_Exception
      */
-    abstract public function changeFolder($id, $displayname, $parent);
+    abstract public function changeFolder($id, $displayname, $parent, $uid = null);
 
     /**
      * Move message
