@@ -7,7 +7,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2012-2013 Horde LLC (http://www.horde.org)
+ * @copyright 2012-2014 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -20,7 +20,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2012-2013 Horde LLC (http://www.horde.org)
+ * @copyright 2012-2014 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -41,6 +41,13 @@ class Horde_ActiveSync_Imap_Adapter
      */
     protected $_defaultNamespace;
 
+    /**
+     * Process id used for logging.
+     *
+     * @var integer
+     */
+    protected $_procid;
+
     /**
      * Cont'r
      *
@@ -53,6 +60,7 @@ class Horde_ActiveSync_Imap_Adapter
         $this->_imap = $params['factory'];
         Horde_Mime_Part::$defaultCharset = 'UTF-8';
         Horde_Mime_Headers::$defaultCharset = 'UTF-8';
+        $this->_procid = getmypid();
     }
 
     /**
@@ -85,7 +93,9 @@ class Horde_ActiveSync_Imap_Adapter
      * @param string $name    The new mailbox name.
      * @param string $parent  The parent mailbox, if any.
      *
-     * @throws Horde_ActiveSync_Exception, Horde_ActiveSync_Exception_FolderExists
+     *  @return string  The new serverid for the mailbox. This is the UTF-8 name
+     *                  of the mailbox. @since 2.9.0
+     *  @throws Horde_ActiveSync_Exception, Horde_ActiveSync_Exception_FolderExists
      */
     public function createMailbox($name, $parent = null)
     {
@@ -104,6 +114,8 @@ class Horde_ActiveSync_Imap_Adapter
             }
             throw new Horde_ActiveSync_Exception($e);
         }
+
+        return $mbox->utf8;
     }
 
     /**
@@ -243,10 +255,7 @@ class Horde_ActiveSync_Imap_Adapter
     }
 
     /**
-     * Return a list of messages from the specified mailbox. If CONDSTORE is NOT
-     * available, or if CONDSTORE IS available, but this is the first request
-     * then the entire message list is returned. Otherwise, only changes since
-     * the last MODSEQ value are taken into consideration.
+     * Return message changes from the specified mailbox.
      *
      * @param Horde_ActiveSync_Folder_Imap $folder  The folder object.
      * @param array $options                        Additional options:
@@ -254,25 +263,27 @@ class Horde_ActiveSync_Imap_Adapter
      *               DEFAULT: 0 (Don't filter).
      *  - protocolversion: (float)  EAS protocol version to support.
      *                     DEFAULT: none REQUIRED
+     *  - softdelete: (boolean)  If true, calculate SOFTDELETE data.
+     *                           @since 2.8.0
      *
      * @return Horde_ActiveSync_Folder_Imap  The folder object, containing any
      *                                       change instructions for the device.
      *
-     * @throws Horde_ActiveSync_Exception_FolderGone, Horde_ActiveSync_Exception
+     * @throws Horde_ActiveSync_Exception_FolderGone,
+     *         Horde_ActiveSync_Exception, Horde_ActiveSync_Exception_StaleState
      */
     public function getMessageChanges(
         Horde_ActiveSync_Folder_Imap $folder, array $options = array())
     {
         $imap = $this->_getImapOb();
         $mbox = new Horde_Imap_Client_Mailbox($folder->serverid());
-        if ($condstore = $imap->queryCapability('CONDSTORE')) {
-            $status_flags = Horde_Imap_Client::STATUS_HIGHESTMODSEQ |
-                Horde_Imap_Client::STATUS_UIDVALIDITY |
-                Horde_Imap_Client::STATUS_UIDNEXT_FORCE;
-        } else {
-            $status_flags = Horde_Imap_Client::STATUS_UIDVALIDITY |
-                Horde_Imap_Client::STATUS_UIDNEXT_FORCE;
-        }
+
+        // Note: non-CONDSTORE servers will return a highestmodseq of 0
+        $status_flags = Horde_Imap_Client::STATUS_HIGHESTMODSEQ |
+            Horde_Imap_Client::STATUS_UIDVALIDITY |
+            Horde_Imap_Client::STATUS_UIDNEXT_FORCE |
+            Horde_Imap_Client::STATUS_MESSAGES |
+            Horde_Imap_Client::STATUS_FORCE_REFRESH;
 
         try {
             $status = $imap->status($mbox, $status_flags);
@@ -280,15 +291,16 @@ class Horde_ActiveSync_Imap_Adapter
             // If we can't status the mailbox, assume it's gone.
             throw new Horde_ActiveSync_Exception_FolderGone($e);
         }
-        if ($condstore) {
-            $modseq = $status[Horde_ActiveSync_Folder_Imap::HIGHESTMODSEQ];
-        } else {
-            $modseq = $status[Horde_ActiveSync_Folder_Imap::HIGHESTMODSEQ] = 0;
-        }
-        $this->_logger->info('IMAP status: ' . print_r($status, true));
+        $this->_logger->info(sprintf(
+            '[%s] IMAP status: %s',
+            $this->_procid,
+            serialize($status))
+        );
         $flags = array();
-        if ($condstore && $folder->modseq() > 0 && $folder->modseq() < $modseq) {
-            $this->_logger->info('CONDSTORE and CHANGES');
+        $modseq = $status[Horde_ActiveSync_Folder_Imap::HIGHESTMODSEQ];
+        if ($modseq && $folder->modseq() > 0 && $folder->modseq() < $modseq) {
+            $this->_logger->info(sprintf(
+                '[%s] CONDSTORE and CHANGES', $this->_procid));
             $folder->checkValidity($status);
             $query = new Horde_Imap_Client_Fetch_Query();
             $query->modseq();
@@ -317,7 +329,7 @@ class Horde_ActiveSync_Imap_Adapter
                             // Ignore, it's out of the FILTERTYPE range.
                             $this->_logger->info(sprintf(
                                 '[%s] Ignoring UID %s since it is outside of the FILTERTYPE (%s)',
-                                getmypid(),
+                                $this->_procid,
                                 $uid,
                                 $headers->getValue('Date')));
                             continue;
@@ -345,22 +357,54 @@ class Horde_ActiveSync_Imap_Adapter
                 throw new Horde_ActiveSync_Exception($e);
             }
             $folder->setRemoved($deleted->ids);
+
+            if (!empty($options['softdelete']) && !empty($options['sincedate'])) {
+                $this->_logger->info(sprintf(
+                    '[%s] Polling for SOFTDELETE in %s before %d',
+                    $this->_procid, $folder->serverid(), $options['sincedate']));
+
+                $query = new Horde_Imap_Client_Search_Query();
+                $query->dateSearch(
+                    new Horde_Date($options['sincedate']),
+                    Horde_Imap_Client_Search_Query::DATE_BEFORE);
+                $query->ids(new Horde_Imap_Client_Ids($folder->messages()));
+                try {
+                    $search_ret = $imap->search(
+                        $mbox,
+                        $query,
+                        array('results' => array(Horde_Imap_Client::SEARCH_RESULTS_MATCH)));
+                } catch (Horde_Imap_Client_Exception $e) {
+                    $this->_logger->err($e->getMessage());
+                    throw new Horde_ActiveSync_Exception($e);
+                }
+                if ($search_ret['count']) {
+                    $this->_logger->info(sprintf(
+                        '[%s] Found %d messages to SOFTDELETE.',
+                        $this->_procid, count($search_ret['match']->ids)));
+                    $folder->setSoftDeleted($search_ret['match']->ids);
+                } else {
+                     $this->_logger->info(sprintf(
+                        '[%s] Found NO messages to SOFTDELETE.',
+                        $this->_procid));
+                }
+                $folder->setSoftDeleteTimes($options['sincedate'], time());
+            }
+
         } elseif ($folder->uidnext() == 0) {
-            $this->_logger->info('INITIAL SYNC');
+            $this->_logger->info(sprintf(
+                '[%s] INITIAL SYNC', $this->_procid));
             $query = new Horde_Imap_Client_Search_Query();
             if (!empty($options['sincedate'])) {
                 $query->dateSearch(
                     new Horde_Date($options['sincedate']),
                     Horde_Imap_Client_Search_Query::DATE_SINCE);
             }
-            $query->flag(Horde_Imap_Client::FLAG_DELETED, false);
             $search_ret = $imap->search(
                 $mbox,
                 $query,
                 array('results' => array(Horde_Imap_Client::SEARCH_RESULTS_MATCH)));
-
-            if ($condstore && $folder->modseq() > 0 && count($search_ret['match']->ids)) {
-                $folder->setChanges($search_ret['match']->ids);
+            if ($modseq && $folder->modseq() > 0 && $search_ret['count']) {
+                $folder->setChanges($search_ret['match']->ids, array());
             } elseif (count($search_ret['match']->ids)) {
                 $query = new Horde_Imap_Client_Fetch_Query();
                 $query->flags();
@@ -375,8 +419,10 @@ class Horde_ActiveSync_Imap_Adapter
                 }
                 $folder->setChanges($search_ret['match']->ids, $flags);
             }
-        } elseif (!$condstore || ($condstore && $modseq == 0)) {
-            $this->_logger->info('NO CONDSTORE or per mailbox MODSEQ: minuid=' . $folder->minuid());
+        } elseif ($modseq == 0) {
+            $this->_logger->info(sprintf(
+                '[%s] NO CONDSTORE or per mailbox MODSEQ. minuid: %s, total_messages: %s',
+                $this->_procid, $folder->minuid(), $status['messages']));
             $folder->checkValidity($status);
             $query = new Horde_Imap_Client_Search_Query();
             if (!empty($options['sincedate'])) {
@@ -384,17 +430,26 @@ class Horde_ActiveSync_Imap_Adapter
                     new Horde_Date($options['sincedate']),
                     Horde_Imap_Client_Search_Query::DATE_SINCE);
             }
-            $query->flag(Horde_Imap_Client::FLAG_DELETED, false);
-            $search_ret = $imap->search(
-                $mbox,
-                $query,
-                array('results' => array(Horde_Imap_Client::SEARCH_RESULTS_MATCH)));
+            try {
+                $search_ret = $imap->search(
+                    $mbox,
+                    $query,
+                    array('results' => array(Horde_Imap_Client::SEARCH_RESULTS_MATCH)));
+            } catch (Horde_Imap_Client_Exception $e) {
+                $this->_logger->err($e->getMessage());
+                throw new Horde_ActiveSync_Exception($e);
+            }
 
             if (count($search_ret['match']->ids)) {
                 // Update flags.
                 $query = new Horde_Imap_Client_Fetch_Query();
                 $query->flags();
-                $fetch_ret = $imap->fetch($mbox, $query, array('ids' => $search_ret['match']));
+                try {
+                    $fetch_ret = $imap->fetch($mbox, $query, array('ids' => $search_ret['match']));
+                } catch (Horde_Imap_Client_Exception $e) {
+                    $this->_logger->err($e->getMessage());
+                    throw new Horde_ActiveSync_Exception($e);
+                }
                 foreach ($fetch_ret as $uid => $data) {
                     $flags[$uid] = array(
                         'read' => (array_search(Horde_Imap_Client::FLAG_SEEN, $data->getFlags()) !== false) ? 1 : 0
@@ -406,6 +461,8 @@ class Horde_ActiveSync_Imap_Adapter
                 $folder->setChanges($search_ret['match']->ids, $flags);
             }
             $folder->setRemoved($imap->vanished($mbox, null, array('ids' => new Horde_Imap_Client_Ids($folder->messages())))->ids);
+        } elseif ($modseq > 0 && $folder->modseq() == 0) {
+                throw new Horde_ActiveSync_Exception_StaleState('Transition to MODSEQ enabled server');
         }
         $folder->setStatus($status);
 
@@ -566,13 +623,11 @@ class Horde_ActiveSync_Imap_Adapter
         $imap = $this->_getImapOb();
         $mbox = new Horde_Imap_Client_Mailbox($folder->serverid());
 
-        // Check for CONDSTORE
-        if ($condstore = $imap->queryCapability('CONDSTORE')) {
-            $status_flags = Horde_Imap_Client::STATUS_HIGHESTMODSEQ |
-                Horde_Imap_Client::STATUS_UIDNEXT_FORCE;
-        } else {
-            $status_flags = Horde_Imap_Client::STATUS_UIDNEXT_FORCE;
-        }
+        // Note: non-CONDSTORE servers will return a highestmodseq of 0
+        $status_flags = Horde_Imap_Client::STATUS_HIGHESTMODSEQ |
+            Horde_Imap_Client::STATUS_UIDNEXT_FORCE |
+            Horde_Imap_Client::STATUS_MESSAGES |
+            Horde_Imap_Client::STATUS_FORCE_REFRESH;
 
         // Get IMAP status.
         try {
@@ -585,14 +640,19 @@ class Horde_ActiveSync_Imap_Adapter
             throw new Horde_ActiveSync_Exception($e);
         }
 
+        // If we have per mailbox MODSEQ then we can pick up flag changes too.
+        $modseq = $status[Horde_ActiveSync_Folder_Imap::HIGHESTMODSEQ];
+        if ($modseq && $folder->modseq() > 0 && $folder->modseq() < $modseq) {
+            return true;
+        }
+
         // Increase in UIDNEXT is always a positive PING.
         if ($folder->uidnext() < $status['uidnext']) {
             return true;
         }
 
-        // If we have per mailbox MODSEQ then we can pick up flag changes too.
-        if ($condstore && $folder->modseq() > 0 &&
-            $folder->modseq() < $status[Horde_ActiveSync_Folder_Imap::HIGHESTMODSEQ]) {
+        // If the message count changes, something certainly changed.
+        if ($folder->total_messages() != $status['messages']) {
             return true;
         }
 
@@ -609,7 +669,7 @@ class Horde_ActiveSync_Imap_Adapter
      */
     public function queryMailbox($query)
     {
-        return $this->_doQuery($query['query'], $query['range']);
+        return $this->_doQuery($query['query']);
     }
 
     /**
@@ -619,6 +679,8 @@ class Horde_ActiveSync_Imap_Adapter
      * @param string $new     The new mailbox name.
      * @param string $parent  The parent mailbox, if any.
      *
+     * @return string  The new serverid for the mailbox.
+     *                 @since 2.9.0
      * @throws Horde_ActiveSync_Exception
      */
     public function renameMailbox($old, $new, $parent = null)
@@ -633,14 +695,17 @@ class Horde_ActiveSync_Imap_Adapter
             $ns = $this->_defaultNamespace();
             $new = $parent . $ns['delimiter'] . $new;
         }
+        $new_mbox = new Horde_Imap_Client_Mailbox($new);
         try {
             $imap->renameMailbox(
                 new Horde_Imap_Client_Mailbox($old),
-                new Horde_Imap_Client_Mailbox($new)
+                $new_mbox
             );
         } catch (Horde_Imap_Client_Exception $e) {
             throw new Horde_ActiveSync_Exception($e);
         }
+
+        return $new_mbox->utf8;
     }
 
     /**
@@ -829,10 +894,14 @@ class Horde_ActiveSync_Imap_Adapter
             $eas_message->displayto = $eas_message->to;
         }
 
+        // Ensure we don't send broken UTF8 data to the client. It makes clients
+        // angry. And we don't like angry clients.
+        $hdr_charset = $imap_message->getStructure()->getHeaderCharset();
+
         // Fill in other header data
         $eas_message->from = $imap_message->getFromAddress();
-        $eas_message->subject = $imap_message->getSubject();
-        $eas_message->threadtopic = $imap_message->getSubject();
+        $eas_message->subject = $this->_validateUtf8($imap_message->getSubject(), $hdr_charset);
+        $eas_message->threadtopic = $eas_message->subject;
         $eas_message->datereceived = $imap_message->getDate();
         $eas_message->read = $imap_message->getFlag(Horde_Imap_Client::FLAG_SEEN);
         $eas_message->cc = $imap_message->getCc();
@@ -855,13 +924,11 @@ class Horde_ActiveSync_Imap_Adapter
         $eas_message->importance = $this->_getEASImportance($priority);
 
         // Get the body data and ensure we have something to send.
-        $message_body_data = $imap_message->getMessageBodyData($options);
+        $message_body_data = $this->_validateMessageBodyData($imap_message->getMessageBodyData($options));
+
         if ($version == Horde_ActiveSync::VERSION_TWOFIVE) {
-            $eas_message->body = $this->_validateUtf8(
-                $message_body_data['plain']['body'],
-                $message_body_data['plain']['charset']
-            );
-            $eas_message->bodysize = Horde_String::length($eas_message->body);
+            $eas_message->body = $message_body_data['plain']['body']->stream;
+            $eas_message->bodysize = $message_body_data['plain']['body']->length(true);
             $eas_message->bodytruncated = $message_body_data['plain']['truncated'];
             $eas_message->attachments = $imap_message->getAttachments($version);
         } else {
@@ -878,7 +945,8 @@ class Horde_ActiveSync_Imap_Adapter
                  ($options['mimesupport'] == Horde_ActiveSync::MIME_SUPPORT_SMIME &&
                   $imap_message->isSigned()))) {
 
-                $this->_logger->info('Sending MIME Message.');
+                $this->_logger->info(sprintf(
+                    '[%s] Sending MIME Message.', $this->_procid));
                 // ActiveSync *REQUIRES* all data sent to be in UTF-8, so we
                 // must convert the body parts to UTF-8. Unfortunately if the
                 // email is signed (or encrypted for that matter) we can't
@@ -894,11 +962,7 @@ class Horde_ActiveSync_Imap_Adapter
                     if (!empty($message_body_data['plain'])) {
                         $plain_mime = new Horde_Mime_Part();
                         $plain_mime->setType('text/plain');
-                        $message_body_data['plain']['body'] = $this->_validateUtf8(
-                            $message_body_data['plain']['body'],
-                            $message_body_data['plain']['charset']
-                        );
-                        $plain_mime->setContents($message_body_data['plain']['body']);
+                        $plain_mime->setContents($message_body_data['plain']['body']->stream, array('usestream' => true));
                         $plain_mime->setCharset('UTF-8');
                         $mime->addPart($plain_mime);
                     }
@@ -907,11 +971,7 @@ class Horde_ActiveSync_Imap_Adapter
                     if (!empty($message_body_data['html'])) {
                         $html_mime = new Horde_Mime_Part();
                         $html_mime->setType('text/html');
-                        $message_body_data['html']['body'] = $this->_validateUtf8(
-                            $message_body_data['html']['body'],
-                            $message_body_data['html']['charset']
-                        );
-                        $html_mime->setContents($message_body_data['html']['body']);
+                        $html_mime->setContents($message_body_data['html']['body']->stream, array('usestream' => true));
                         $html_mime->setCharset('UTF-8');
                         $mime->addPart($html_mime);
                     }
@@ -951,7 +1011,8 @@ class Horde_ActiveSync_Imap_Adapter
                 // MIME Truncation
                 $airsync_body->type = Horde_ActiveSync::BODYPREF_TYPE_MIME;
                 $this->_logger->info(sprintf(
-                    'Checking MIMETRUNCATION: %s, ServerData: %s',
+                    '[%s] Checking MIMETRUNCATION: %s, ServerData: %s',
+                    $this->_procid,
                     $options['truncation'],
                     $airsync_body->estimateddatasize));
                 if (!empty($options['truncation']) &&
@@ -966,37 +1027,32 @@ class Horde_ActiveSync_Imap_Adapter
                       isset($options['bodyprefs'][Horde_ActiveSync::BODYPREF_TYPE_RTF])) {
 
                 // Sending non MIME encoded HTML message text.
-                $this->_logger->info('Sending HTML Message.');
+                $this->_logger->info(sprintf(
+                    '[%s] Sending HTML Message.',
+                    $this->_procid));
                 if (empty($message_body_data['html'])) {
                     $airsync_body->type = Horde_ActiveSync::BODYPREF_TYPE_PLAIN;
                     $message_body_data['html'] = array(
                         'body' => $message_body_data['plain']['body'],
                         'estimated_size' => $message_body_data['plain']['size'],
-                        'truncated' => $message_body_data['plain']['truncated'],
-                        'charset' => $message_body_data['plain']['charset']
+                        'truncated' => $message_body_data['plain']['truncated']
                     );
                 } else {
                     $airsync_body->type = Horde_ActiveSync::BODYPREF_TYPE_HTML;
                 }
                 $airsync_body->estimateddatasize = $message_body_data['html']['estimated_size'];
                 $airsync_body->truncated = $message_body_data['html']['truncated'];
-                $airsync_body->data = $this->_validateUtf8(
-                    $message_body_data['html']['body'],
-                    $message_body_data['html']['charset']
-                );
+                $airsync_body->data = $message_body_data['html']['body']->stream;
                 $eas_message->airsyncbasebody = $airsync_body;
                 $eas_message->airsyncbaseattachments = $imap_message->getAttachments($version);
             } elseif (isset($options['bodyprefs'][Horde_ActiveSync::BODYPREF_TYPE_PLAIN])) {
 
                 // Non MIME encoded plaintext
-                $this->_logger->info('Sending PLAINTEXT Message.');
-                $message_body_data['plain']['body'] = $this->_validateUtf8(
-                    $message_body_data['plain']['body'],
-                    $message_body_data['plain']['charset']
-                );
+                $this->_logger->info(sprintf(
+                    '[%s] Sending PLAINTEXT Message.', $this->_procid));
                 $airsync_body->estimateddatasize = $message_body_data['plain']['size'];
                 $airsync_body->truncated = $message_body_data['plain']['truncated'];
-                $airsync_body->data = $message_body_data['plain']['body'];
+                $airsync_body->data = $message_body_data['plain']['body']->stream;
                 $airsync_body->type = Horde_ActiveSync::BODYPREF_TYPE_PLAIN;
                 $eas_message->airsyncbasebody = $airsync_body;
                 $eas_message->airsyncbaseattachments = $imap_message->getAttachments($version);
@@ -1005,14 +1061,9 @@ class Horde_ActiveSync_Imap_Adapter
 
         // Preview?
         if ($version >= Horde_ActiveSync::VERSION_FOURTEEN && !empty($options['bodyprefs']['preview'])) {
-            $eas_message->airsyncbasebody->preview = Horde_String::substr(
-                $message_body_data['plain']['body'],
-                0,
-                $options['bodyprefs']['preview'],
-                $message_body_data['plain']['charset']);
-            $eas_message->airsyncbasebody->preview = $this->_validateUtf8(
-                $eas_message->airsyncbasebody->preview,
-                $message_body_data['plain']['charset']);
+            $message_body_data['plain']['body']->rewind();
+            $eas_message->airsyncbasebody->preview =
+                $message_body_data['plain']['body']->substring(0, $options['bodyprefs']['preview']);
         }
 
         // Check for special message types.
@@ -1197,7 +1248,7 @@ class Horde_ActiveSync_Imap_Adapter
      *
      * @throws Horde_ActiveSync_Exception
      */
-    protected function _doQuery(array $query, $range = null)
+    protected function _doQuery(array $query)
     {
         $imap_query = new Horde_Imap_Client_Search_Query();
         $mboxes = array();
@@ -1243,7 +1294,11 @@ class Horde_ActiveSync_Imap_Adapter
         }
         foreach ($mboxes as $mbox) {
             try {
-                $search_res = $this->_getImapOb()->search($mbox, $imap_query);
+                $search_res = $this->_getImapOb()->search(
+                    $mbox,
+                    $imap_query,
+                    array('results' => array(Horde_Imap_Client::SEARCH_RESULTS_MATCH, Horde_Imap_Client::SEARCH_RESULTS_SAVE, Horde_Imap_Client::SEARCH_RESULTS_COUNT))
+                );
             } catch (Horde_Imap_Client_Exception $e) {
                 throw new Horde_ActiveSync_Exception($e);
             }
@@ -1412,7 +1467,8 @@ class Horde_ActiveSync_Imap_Adapter
             return $imap->fetch($mbox, $query, array('ids' => $ids));
         } catch (Horde_Imap_Client_Exception $e) {
             $this->_logger->err(sprintf(
-                'Unable to fetch message: %s',
+                '[%s] Unable to fetch message: %s',
+                $this->_procid,
                 $e->getMessage()));
             throw new Horde_ActiveSync_Exception($e);
         }
@@ -1464,10 +1520,15 @@ class Horde_ActiveSync_Imap_Adapter
      */
     protected function _validateUtf8($data, $from_charset)
     {
-        $this->_logger->info('Validating UTF-8 data coming from ' . $from_charset);
+        $this->_logger->info(sprintf(
+            '[%s] Validating UTF-8 data coming from %s',
+            $this->_procid,
+            $from_charset));
         $text = Horde_String::convertCharset($data, $from_charset, 'UTF-8');
         if (!Horde_String::validUtf8($text)) {
-            $this->_logger->info('Found invalid UTF-8 data, try different encodings.');
+            $this->_logger->info(sprintf(
+                '[%s] Found invalid UTF-8 data, try different encodings.',
+                $this->_procid));
             $test_charsets = array(
                 'windows-1252',
                 'UTF-8'
@@ -1476,7 +1537,10 @@ class Horde_ActiveSync_Imap_Adapter
                 if ($charset != $from_charset) {
                     $text = Horde_String::convertCharset($data, $charset, 'UTF-8');
                     if (Horde_String::validUtf8($text)) {
-                        $this->_logger->info('Found valid UTF-8 data when using ' . $charset);
+                        $this->_logger->info(sprintf(
+                            '[%s] Found valid UTF-8 data when using %s',
+                            $this->_procid,
+                            $charset));
                         return $text;
                     }
                 }
@@ -1486,7 +1550,9 @@ class Horde_ActiveSync_Imap_Adapter
             // that fails, force a conersion to UTF-8 as a last resort. Need
             // to break string into smaller chunks to avoid hitting
             // https://bugs.php.net/bug.php?id=37793
-            $this->_logger->info('Could not encode UTF-8 data. Removing non 7-bit characters.');
+            $this->_logger->info(sprintf(
+                '[%s] Could not encode UTF-8 data. Removing non 7-bit characters.',
+                $this->_procid));
             $chunk_size = 4000;
             $text = '';
             while ($data !== false && strlen($data)) {
@@ -1503,4 +1569,36 @@ class Horde_ActiveSync_Imap_Adapter
         return $text;
     }
 
-}
\ No newline at end of file
+    /**
+     * Converts and validates the message body data structure.
+     *
+     * @param array $data  Message body data structure.
+     *
+     * @return array  The message body data structure, with the [html][body] and
+     *                [plain][body] data converted to UTF-8, EOL normalized and
+     *                placed in a stream.
+     */
+    protected function _validateMessageBodyData($data)
+    {
+        //We will need the eol filter to work around PHP bug 65776.
+        stream_filter_register('horde_eol', 'Horde_Stream_Filter_Eol');
+
+        if (!empty($data['plain'])) {
+            $stream = new Horde_Stream_Temp(array('max_memory' => 1048576));
+            $filter_h = stream_filter_append($stream->stream, 'horde_eol', STREAM_FILTER_WRITE);
+            $stream->add($this->_validateUtf8($data['plain']['body'], $data['plain']['charset']), true);
+            stream_filter_remove($filter_h);
+            $data['plain']['body'] = $stream;
+        }
+        if (!empty($data['html'])) {
+            $stream = new Horde_Stream_Temp(array('max_memory' => 1048576));
+            $filter_h = stream_filter_append($stream->stream, 'horde_eol', STREAM_FILTER_WRITE);
+            $stream->add($this->_validateUtf8($data['html']['body'], $data['html']['charset']), true);
+            stream_filter_remove($filter_h);
+            $data['html']['body'] = $stream;
+        }
+
+        return $data;
+    }
+
+}
