@@ -7,7 +7,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2012-2013 Horde LLC (http://www.horde.org)
+ * @copyright 2012-2014 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -22,7 +22,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2012-2013 Horde LLC (http://www.horde.org)
+ * @copyright 2012-2014 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -209,7 +209,9 @@ class Horde_ActiveSync_Imap_Message
      */
     public function getMessageBodyData(array $options = array())
     {
-        $version = empty($options['protocolversion']) ? 2.5 : $options['protocolversion'];
+        $version = empty($options['protocolversion']) ?
+            Horde_ActiveSync::VERSION_TWOFIVE :
+            $options['protocolversion'];
 
         // Look for the parts we need. We try to detect and fetch only the parts
         // we need, while ensuring we have something to return. So, e.g., if we
@@ -365,7 +367,7 @@ class Horde_ActiveSync_Imap_Message
             }
 
             if ($version >= Horde_ActiveSync::VERSION_TWELVE &&
-                !($truncated && $options['bodyprefs'][Horde_ActiveSync::BODYPREF_TYPE_HTML]['allornone'])) {
+                !($truncated && !empty($options['bodyprefs'][Horde_ActiveSync::BODYPREF_TYPE_HTML]['allornone']))) {
                 $return['html'] = array(
                     'charset' => $html_charset,
                     'body' => $html,
@@ -389,30 +391,12 @@ class Horde_ActiveSync_Imap_Message
     {
         $ret = array();
         $map = $this->_message->contentTypeMap();
-        $headers = $this->getHeaders();
-        $charset = $this->_message->getHeaderCharset();
         foreach ($map as $id => $type) {
             if ($this->isAttachment($id, $type)) {
-                $mime_part = $this->getMimePart($id, array('nocontents' => true));
-                if ($version > Horde_ActiveSync::VERSION_TWOFIVE) {
-                    $atc = Horde_ActiveSync::messageFactory('AirSyncBaseAttachment');
-                    $atc->contentid = $mime_part->getContentId();
-                    $atc->isinline = $mime_part->getDisposition() == 'inline';
-                } else {
-                    $atc = Horde_ActiveSync::messageFactory('Attachment');
-                    $atc->attoid = $mime_part->getContentId();
+                if ($type != 'application/ms-tnef' || (!$mime_part = $this->_decodeTnefData($id))) {
+                    $mime_part = $this->getMimePart($id, array('nocontents' => true));
                 }
-                $atc->attsize = $mime_part->getBytes();
-                $atc->attname = $this->_mbox . ':' . $this->_uid . ':' . $id;
-                $atc->displayname = Horde_String::convertCharset(
-                    $this->getPartName($mime_part, true),
-                    $charset,
-                    'UTF-8',
-                    true);
-                $atc->attmethod = in_array($mime_part->getType(), array('message/rfc822', 'message/disposition-notification'))
-                    ? Horde_ActiveSync_Message_AirSyncBaseAttachment::ATT_TYPE_EMBEDDED
-                    : Horde_ActiveSync_Message_AirSyncBaseAttachment::ATT_TYPE_NORMAL;
-                $ret[] = $atc;
+                $ret[] = $this->_buildEasAttachmentFromMime($id, $mime_part, $version);
             }
         }
 
@@ -420,7 +404,85 @@ class Horde_ActiveSync_Imap_Message
     }
 
     /**
-     * Return an array of mime parts for each message attachment.
+     * Build an appropriate attachment object from the given mime part.
+     *
+     * @param integer $id                  The mime id for the part
+     * @param Horde_Mime_Part  $mime_part  The mime part.
+     * @param float $version               The EAS version.
+     *
+     * @return Horde_ActiveSync_Message_AirSyncBaseAttachment |
+     *         Horde_ActiveSync_Message_Attachment
+     */
+    protected function _buildEasAttachmentFromMime($id, Horde_Mime_Part $mime_part, $version)
+    {
+        if ($version > Horde_ActiveSync::VERSION_TWOFIVE) {
+            $atc = Horde_ActiveSync::messageFactory('AirSyncBaseAttachment');
+            $atc->contentid = $mime_part->getContentId();
+            $atc->isinline = $mime_part->getDisposition() == 'inline';
+        } else {
+            $atc = Horde_ActiveSync::messageFactory('Attachment');
+            $atc->attoid = $mime_part->getContentId();
+        }
+        $atc->attsize = intval($mime_part->getBytes(true));
+        $atc->attname = $this->_mbox . ':' . $this->_uid . ':' . $id;
+        $atc->displayname = Horde_String::convertCharset(
+            $this->getPartName($mime_part, true),
+            $this->_message->getHeaderCharset(),
+            'UTF-8',
+            true);
+        $atc->attmethod = in_array($mime_part->getType(), array('message/rfc822', 'message/disposition-notification'))
+            ? Horde_ActiveSync_Message_AirSyncBaseAttachment::ATT_TYPE_EMBEDDED
+            : Horde_ActiveSync_Message_AirSyncBaseAttachment::ATT_TYPE_NORMAL;
+
+        return $atc;
+    }
+
+    /**
+     * Convert a TNEF attachment into a multipart/mixed part.
+     *
+     * @param  integer|Horde_Mime_part $data  Either a mime part id or a
+     *                                        Horde_Mime_Part object containing
+     *                                        the TNEF attachment.
+     *
+     * @return Horde_Mime_Part  The multipart/mixed MIME part containing any
+     *                          attachment data we can decode.
+     */
+    protected function _decodeTnefData($data)
+    {
+        $wrapper = new Horde_Mime_Part();
+        $wrapper->setType('multipart/mixed');
+
+        if (!($data instanceof Horde_Mime_Part)) {
+            $mime_part = $this->getMimePart($data);
+        } else {
+            $mime_part = $data;
+        }
+        $tnef_parser = Horde_Compress::factory('Tnef');
+        $tnef_data = $tnef_parser->decompress($mime_part->getContents());
+        if (!count($tnef_data)) {
+            return false;
+        }
+
+        reset($tnef_data);
+        while (list(,$data) = each($tnef_data)) {
+            $tmp_part = new Horde_Mime_Part();
+            $tmp_part->setName($data['name']);
+            $tmp_part->setDescription($data['name']);
+            $tmp_part->setContents($data['stream']);
+
+            $type = $data['type'] . '/' . $data['subtype'];
+            if (in_array($type, array('application/octet-stream', 'application/base64'))) {
+                $type = Horde_Mime_Magic::filenameToMIME($data['name']);
+            }
+            $tmp_part->setType($type);
+            $wrapper->addPart($tmp_part);
+        }
+
+        return $wrapper;
+    }
+
+    /**
+     * Return an array of mime parts for each messages attachment.
      *
      * @return array An array of Horde_Mime_Part objects.
      */
@@ -430,7 +492,15 @@ class Horde_ActiveSync_Imap_Message
         $map = $this->_message->contentTypeMap();
         foreach ($map as $id => $type) {
             if ($this->isAttachment($id, $type)) {
-                $mime_parts[] = $this->getMimePart($id);
+                $mpart = $this->getMimePart($id);
+                if ($mpart->getType() == 'text/calendar') {
+                    $mpart->setDisposition('inline');
+                }
+                if ($mpart->getType() != 'application/ms-tnef' ||
+                    ($mpart->getType() == 'application/ms-tnef' && !$part = $this->_decodeTnefData($mpart))) {
+                    $part = $mpart;
+                }
+                $mime_parts[] = $part;
             }
         }
 
@@ -734,7 +804,6 @@ class Horde_ActiveSync_Imap_Message
                 return true;
             }
             return false;
-        case 'application/ms-tnef':
         case 'application/pkcs7-signature':
             return false;
         }
@@ -797,13 +866,35 @@ class Horde_ActiveSync_Imap_Message
     /**
      * Return the S/MIME status of this message (RFC2633)
      *
-     * @return boolean True if message is S/MIME signed, false otherwise.
+     * @param Horde_Mime_Part $message  A mime part to check. If omitted, use
+     *                                  $this->_message.
+     *
+     * @return boolean True if message is S/MIME signed or encrypted,
+     *                 false otherwise.
      */
-    public function isSigned()
+    public function isSigned($message = null)
     {
-        return $this->_message->getSubType() == 'signed' ||
-               ($this->_message->getType() == 'application/pkcs7-mime' &&
-                $this->_message->getContentTypeParameter('smime-type') == 'signed-data');
+        if (empty($message)) {
+            $message = $this->_message;
+        }
+        if ($message->getType() == 'application/pkcs7-mime') {
+            return true;
+        }
+
+        if ($message->getPrimaryType() == 'multipart') {
+            if ($message->getSubType() == 'signed') {
+                return true;
+            }
+
+            // Signed/encrypted part might be lower in the mime structure
+            foreach ($message->getParts() as $part) {
+                if ($this->isSigned($part)) {
+                    return true;
+                }
+            }
+        }
+
+        return false;
     }
 
 }
