@@ -18,7 +18,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2009-2013 Horde LLC (http://www.horde.org)
+ * @copyright 2009-2014 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -31,7 +31,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2009-2013 Horde LLC (http://www.horde.org)
+ * @copyright 2009-2014 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -97,6 +97,13 @@ class Horde_ActiveSync_Message_Base
      */
     protected $_version = Horde_ActiveSync::VERSION_TWOFIVE;
 
+    /**
+     * The device object
+     *
+     * @var Horde_ActiveSync_Device
+     */
+    protected $_device;
+
     /**
      * Const'r
      *
@@ -105,6 +112,7 @@ class Horde_ActiveSync_Message_Base
      *             DEFAULT: none (No logging).
      *   - protocolversion: (float)  The version of EAS to support.
      *              DEFAULT: Horde_ActiveSync::VERSION_TWOFIVE (2.5)
+     *   - device: (Horde_ActiveSync_Device)  The device object. @since
      *
      * @return Horde_ActiveSync_Message_Base
      */
@@ -118,6 +126,9 @@ class Horde_ActiveSync_Message_Base
         if (!empty($options['protocolversion'])) {
             $this->_version = $options['protocolversion'];
         }
+        if (!empty($options['device'])) {
+            $this->_device = $options['device'];
+        }
     }
 
     /**
@@ -172,7 +183,7 @@ class Horde_ActiveSync_Message_Base
     {
         if (!array_key_exists($property, $this->_properties)) {
             $this->_logger->err('Unknown property: ' . $property);
-            throw new InvalidArgumentException('Unknown property: ' . $property);
+            throw new InvalidArgumentException(get_class($this) . ' Unknown property: ' . $property);
         }
         $this->_properties[$property] = $value;
         $this->_exists[$property] = true;
@@ -323,7 +334,7 @@ class Horde_ActiveSync_Message_Base
                         if (isset($map[self::KEY_TYPE])) {
                             // Complex type, decode recursively
                             if ($map[self::KEY_TYPE] == self::TYPE_DATE || $map[self::KEY_TYPE] == self::TYPE_DATE_DASHES) {
-                                $decoded = self::_parseDate($decoder->getElementContent());
+                                $decoded = $this->_parseDate($decoder->getElementContent());
                                 if (!$decoder->getElementEndTag()) {
                                     throw new Horde_ActiveSync_Exception('Missing expected wbxml end tag');
                                 }
@@ -390,28 +401,29 @@ class Horde_ActiveSync_Message_Base
                     $this->$map[self::KEY_ATTRIBUTE]->encodeStream($encoder);
                     $encoder->endTag();
                 } elseif (isset($map[self::KEY_VALUES]) && is_array($this->$map[self::KEY_ATTRIBUTE])) {
-                    // Array of objects
-                    if (!count($this->$map[self::KEY_ATTRIBUTE]) && $this->_checkSendEmpty($tag)) {
-                        // Some array tags must be sent as empty tags.
-                        $encoder->startTag($tag, null, true);
-                    }
-                    $encoder->startTag($tag); // Outputs array container (eg Attachments)
-                    foreach ($this->$map[self::KEY_ATTRIBUTE] as $element) {
-                        if (is_object($element)) {
-                            // Outputs object container (eg Attachment)
-                            $encoder->startTag($map[self::KEY_VALUES]);
-                            $element->encodeStream($encoder);
-                            $encoder->endTag();
-                        } else {
-                            // Do not ever output empty items here
-                            if(strlen($element) > 0) {
+                    // Array of objects. Note that some array values must be
+                    // send as an empty tag if they contain no elements.
+                    if (count($this->$map[self::KEY_ATTRIBUTE])) {
+                        $encoder->startTag($tag);
+                        foreach ($this->$map[self::KEY_ATTRIBUTE] as $element) {
+                            if (is_object($element)) {
+                                // Outputs object container (eg Attachment)
                                 $encoder->startTag($map[self::KEY_VALUES]);
-                                $encoder->content($element);
+                                $element->encodeStream($encoder);
                                 $encoder->endTag();
+                            } else {
+                                // Do not ever output empty items here
+                                if(strlen($element) > 0) {
+                                    $encoder->startTag($map[self::KEY_VALUES]);
+                                    $encoder->content($element);
+                                    $encoder->endTag();
+                                }
                             }
                         }
+                        $encoder->endTag();
+                    } elseif ($this->_checkSendEmpty($tag)) {
+                        $encoder->startTag($tag, null, true);
                     }
-                    $encoder->endTag();
                 } else {
                     // Simple type
                     if (!is_resource($this->$map[self::KEY_ATTRIBUTE]) &&
@@ -438,7 +450,7 @@ class Horde_ActiveSync_Message_Base
                     $encoder->startTag($tag);
                     if (isset($map[self::KEY_TYPE]) && ($map[self::KEY_TYPE] == self::TYPE_DATE || $map[self::KEY_TYPE] == self::TYPE_DATE_DASHES)) {
                         if (!empty($this->$map[self::KEY_ATTRIBUTE])) { // don't output 1-1-1970
-                            $encoder->content(self::_formatDate($this->$map[self::KEY_ATTRIBUTE], $map[self::KEY_TYPE]));
+                            $encoder->content($this->_formatDate($this->$map[self::KEY_ATTRIBUTE], $map[self::KEY_TYPE]));
                         }
                     } elseif (isset($map[self::KEY_TYPE]) && $map[self::KEY_TYPE] == self::TYPE_HEX) {
                         $encoder->content(Horde_String::upper(bin2hex($this->$map[self::KEY_ATTRIBUTE])));
@@ -517,7 +529,7 @@ class Horde_ActiveSync_Message_Base
      *
      * @return string  The formatted date
      */
-    static protected function _formatDate(Horde_Date $dt, $type)
+    protected function _formatDate(Horde_Date $dt, $type)
     {
         if ($type == Horde_ActiveSync_Message_Base::TYPE_DATE) {
             return $dt->setTimezone('UTC')->format('Ymd\THis\Z');
@@ -533,7 +545,7 @@ class Horde_ActiveSync_Message_Base
      *
      * @return Horde_Date  The Horde_Date
      */
-    static protected function _parseDate($ts)
+    protected function _parseDate($ts)
     {
         if (preg_match("/(\d{4})[^0-9]*(\d{2})[^0-9]*(\d{2})(T(\d{2})[^0-9]*(\d{2})[^0-9]*(\d{2})(.\d+)?Z){0,1}$/", $ts, $matches)) {
             return new Horde_Date($ts);
