@@ -16,7 +16,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2010-2013 Horde LLC (http://www.horde.org)
+ * @copyright 2010-2014 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -29,7 +29,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2010-2013 Horde LLC (http://www.horde.org)
+ * @copyright 2010-2014 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  *
@@ -134,7 +134,7 @@ class Horde_ActiveSync_Message_MeetingRequest extends Horde_ActiveSync_Message_B
             }
         }
 
-        $tz = new Horde_ActiveSync_Timezone();
+        $tz = new Horde_Mapi_Timezone();
         $this->timezone = $tz->getSyncTZFromOffsets(
         $tz->getOffsetsFromDate(new Horde_Date()));
         $this->alldayevent = (int)$this->_isAllDay();
@@ -171,7 +171,7 @@ class Horde_ActiveSync_Message_MeetingRequest extends Horde_ActiveSync_Message_B
         } catch (Horde_Icalendar_Exception $e) {}
 
         try {
-            $this->globalobjid = Horde_ActiveSync_Utils::createGoid($vevent->getAttribute('UID'));
+            $this->globalobjid = Horde_Mapi::createGoid($vevent->getAttribute('UID'));
             $this->starttime = new Horde_Date($vevent->getAttribute('DTSTART'));
             $this->endtime = new Horde_Date($vevent->getAttribute('DTEND'));
         } catch (Horde_Exception $e) {
