@@ -7,7 +7,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2012-2013 Horde LLC (http://www.horde.org)
+ * @copyright 2012-2014 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -19,7 +19,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2012-2013 Horde LLC (http://www.horde.org)
+ * @copyright 2012-2014 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -30,21 +30,28 @@ class Horde_ActiveSync_Request_Autodiscover extends Horde_ActiveSync_Request_Bas
      *
      * @return text  The content type of the response (text/xml).
      */
-    public function handle()
+    public function handle(Horde_Controller_Request $request = null)
     {
-        $input_stream = $this->_decoder->getStream();
         $parser = xml_parser_create();
-        xml_parse_into_struct($parser, stream_get_contents($input_stream), $values);
+        xml_parse_into_struct(
+          $parser,
+          $this->_decoder->getStream()->getString(),
+          $values);
+
+        // Get $_SERVER
+        $server = $request->getServerVars();
 
         // Some broken clients *cough* android *cough* don't send the actual
         // XML data structure at all, but instead use the email address as
         // the username in the HTTP_AUTHENTICATION data. There are so many things
         // wrong with this, but try to work around it if we can.
-        if (empty($values) && !empty($_SERVER['HTTP_AUTHORIZATION'])) {
-            $hash = base64_decode(str_replace('Basic ', '', $_SERVER['HTTP_AUTHORIZATION']));
+        if (empty($values) && !empty($server['HTTP_AUTHORIZATION'])) {
+            $hash = base64_decode(str_replace('Basic ', '', $server['HTTP_AUTHORIZATION']));
             if (strpos($hash, ':') !== false) {
                 list($email, $pass) = explode(':', $hash, 2);
             }
+        } elseif (empty($values)) {
+            throw new Horde_Exception_AuthenticationFailure('No username provided.');
         } else {
             $email = $values[2]['value'];
         }
@@ -54,29 +61,27 @@ class Horde_ActiveSync_Request_Autodiscover extends Horde_ActiveSync_Request_Bas
         }
 
         if (!empty($values)) {
-            $params = array('request_schema' => $values[0]['attributes']['XMLNS']);
+            $params = array('request_schema' => trim($values[0]['attributes']['XMLNS']));
             // Response Schema is not in a set place.
             foreach ($values as $value) {
                 if ($value['tag'] == 'ACCEPTABLERESPONSESCHEMA') {
-                    $params['response_schema'] = $value['value'];
+                    $params['response_schema'] = trim($value['value']);
                     break;
                 }
             }
         } else {
-            $params = array();
+            // Assume broken clients want these schemas.
+            $params = array(
+              'request_schema' => 'http://schemas.microsoft.com/exchange/autodiscover/mobilesync/requestschema/2006',
+              'response_schema' => 'http://schemas.microsoft.com/exchange/autodiscover/mobilesync/responseschema/2006'
+            );
         }
         $results = $this->_driver->autoDiscover($params);
-
         if (empty($results['raw_xml'])) {
-            fwrite(
-                $this->_encoder->getStream(),
-                $this->_buildResponseString($results));
+            $this->_encoder->getStream()->add($this->_buildResponseString($results));
         } else {
             // The backend is taking control of the XML.
-            fwrite(
-                $this->_encoder->getStream(),
-                $results['raw_xml']
-            );
+            $this->_encoder->getStream()->add($results['raw_xml']);
         }
 
         return 'text/xml';
