@@ -6,7 +6,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2009-2013 Horde LLC (http://www.horde.org)
+ * @copyright 2009-2014 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -29,7 +29,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2009-2013 Horde LLC (http://www.horde.org)
+ * @copyright 2009-2014 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -69,35 +69,40 @@ class Horde_ActiveSync_Request_MeetingResponse extends Horde_ActiveSync_Request_
         }
         while ($this->_decoder->getElementStartTag(self::MEETINGRESPONSE_REQUEST)) {
             $req = array();
-            if ($this->_decoder->getElementStartTag(self::MEETINGRESPONSE_USERRESPONSE)) {
-                $req['response'] = $this->_decoder->getElementContent();
-                if (!$this->_decoder->getElementEndTag()) {
-                    throw new Horde_ActiveSync_Exception('Protocol Error');
+            while (($tag = ($this->_decoder->getElementStartTag(self::MEETINGRESPONSE_USERRESPONSE) ? self::MEETINGRESPONSE_USERRESPONSE :
+                   ($this->_decoder->getElementStartTag(self::MEETINGRESPONSE_FOLDERID) ? self::MEETINGRESPONSE_FOLDERID :
+                   ($this->_decoder->getElementStartTag(self::MEETINGRESPONSE_REQUESTID) ? self::MEETINGRESPONSE_REQUESTID : -1)))) != -1) {
+
+                switch ($tag) {
+                case self::MEETINGRESPONSE_USERRESPONSE:
+                    $req['response'] = $this->_decoder->getElementContent();
+                    if (!$this->_decoder->getElementEndTag()) {
+                        throw new Horde_ActiveSync_Exception('Protocol Error');
+                    }
+                    break;
+                case self::MEETINGRESPONSE_FOLDERID:
+                    $req['folderid'] = $this->_activeSync->getCollectionsObject()
+                        ->getBackendIdForFolderUid($this->_decoder->getElementContent());
+                    if (!$this->_decoder->getElementEndTag()) {
+                        throw new Horde_ActiveSync_Exception('Protocol Error');
+                    }
+                    break;
+                case self::MEETINGRESPONSE_REQUESTID:
+                    $req['requestid'] = $this->_decoder->getElementContent();
+                    if (!$this->_decoder->getElementEndTag()) {
+                        throw new Horde_ActiveSync_Exception('Protocol Error');
+                    }
+                    break;
                 }
             }
-
-            if ($this->_decoder->getElementStartTag(self::MEETINGRESPONSE_FOLDERID)) {
-                $req['folderid'] = $this->_activeSync->getCollectionsObject()
-                    ->getBackendIdForFolderUid(
-                        $this->_decoder->getElementContent());
-                if (!$this->_decoder->getElementEndTag()) {
-                    throw new Horde_ActiveSync_Exception('Protocol Error');
-                }
-            }
-
-            if ($this->_decoder->getElementStartTag(self::MEETINGRESPONSE_REQUESTID)) {
-                $req['requestid'] = $this->_decoder->getElementContent();
-                if (!$this->_decoder->getElementEndTag()) {
-                    throw new Horde_ActiveSync_Exception('Protocol Error');
-                }
-            }
-
+            $requests[] = $req;
+            // </self::MEETINGRESPONSE_REQUEST>
             if (!$this->_decoder->getElementEndTag()) {
                 throw new Horde_ActiveSync_Exception('Protocol Error');
             }
-            $requests[] = $req;
         }
 
+        // </self::MEETINGRESPONSE>
         if (!$this->_decoder->getElementEndTag()) {
             throw new Horde_ActiveSync_Exception('Protocol Error');
         }
@@ -114,7 +119,22 @@ class Horde_ActiveSync_Request_MeetingResponse extends Horde_ActiveSync_Request_
             } catch (Horde_Exception_NotFound $e) {
                 $status = self::STATUS_SERVER_ERROR;
             } catch (Horde_ActiveSync_Exception $e) {
-                $status = self::STATUS_INVALID_REQUEST;
+                // Outlook seems to sometimes send the response from the
+                // calendar folder instead of the mailbox regardless of where
+                // the message is replied to from this will obviously fail,
+                // so we should try one last time to get the folder from the
+                // INBOX. If it was moved to some other mail folder, we have to
+                // just give up.
+                $this->_logger->info(sprintf('[%s] Trying to find meeting request in INBOX.', $this->_procid));
+                $req['folderid'] = 'INBOX';
+                try {
+                    $uid = $this->_driver->meetingResponse($req);
+                    $status = self::STATUS_SUCCESS;
+                    $this->_logger->info(sprintf('[%s] Successfully found meeting response in INBOX.', $this->_procid));
+                } catch (Horde_ActiveSync_Exception $e) {
+                    $this->_logger->err(sprintf('[%s] Meeting request unable to be located.', $this->_procid));
+                    $status = self::STATUS_INVALID_REQUEST;
+                }
             }
 
             $this->_encoder->startTag(self::MEETINGRESPONSE_RESULT);
@@ -136,4 +156,5 @@ class Horde_ActiveSync_Request_MeetingResponse extends Horde_ActiveSync_Request_
 
         return true;
     }
+
 }
\ No newline at end of file
