@@ -18,7 +18,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2009-2013 Horde LLC (http://www.horde.org)
+ * @copyright 2009-2014 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -30,7 +30,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2009-2013 Horde LLC (http://www.horde.org)
+ * @copyright 2009-2014 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -105,7 +105,9 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
         if ($this->_device->version >= Horde_ActiveSync::VERSION_TWELVEONE) {
             // We don't have a previous FOLDERSYNC.
             if (!$this->_collections->haveHierarchy()) {
-                $this->_logger->debug('No HIERARCHY SYNCKEY in sync_cache, invalidating.');
+                $this->_logger->notice(sprintf(
+                    '[%s] No HIERARCHY SYNCKEY in sync_cache, invalidating.',
+                    $this->_procid));
                 $this->_statusCode = self::STATUS_FOLDERSYNC_REQUIRED;
                 $this->_handleGlobalSyncError();
                 return true;
@@ -116,10 +118,10 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
         if (!$this->_decoder->getElementStartTag(Horde_ActiveSync::SYNC_SYNCHRONIZE)) {
             if ($this->_device->version >= Horde_ActiveSync::VERSION_TWELVEONE) {
                 $this->_logger->info(sprintf(
-                    '[%s] Empty Sync request taking info from SyncCache.',
+                    '[%s] Empty Sync request, taking info from SyncCache.',
                     $this->_procid));
                 if ($this->_collections->cachedCollectionCount() == 0) {
-                    $this->_logger->err(sprintf(
+                    $this->_logger->warn(sprintf(
                         '[%s] Empty SYNC request but no SyncCache or SyncCache with no collections.',
                         $this->_procid));
                     $this->_statusCode = self::STATUS_REQUEST_INCOMPLETE;
@@ -129,7 +131,7 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                     $this->_collections->loadCollectionsFromCache();
                     $csk = $this->_collections->confirmed_synckeys;
                     if (count($csk) > 0) {
-                        $this->_logger->err(sprintf(
+                        $this->_logger->warn(sprintf(
                             '[%s] Unconfirmed synckeys, but handling a short request. Request full SYNC.',
                             $this->_procid));
                         $this->_statusCode = self::STATUS_REQUEST_INCOMPLETE;
@@ -218,7 +220,9 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                 if ($this->_collections->hbinterval !== false &&
                     $this->_collections->wait !== false) {
 
-                    $this->_logger->err('Received both HBINTERVAL and WAIT interval in same request. VIOLATION.');
+                    $this->_logger->err(sprintf(
+                        '[%s] Received both HBINTERVAL and WAIT interval in same request.',
+                        $this->_procid));
                     $this->_statusCode = Horde_ActiveSync_Status::INVALID_XML;
                     $this->_handleGlobalSyncError();
                     return true;
@@ -228,12 +232,6 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                 $this->_collections->validateFromCache();
             }
 
-            // Check for inifinite sync loops.
-            if (!$this->_collections->checkLoopCounters()) {
-                $this->_statusCode = Horde_ActiveSync_Status::SERVER_ERROR;
-                $this->_handleGlobalSyncError();
-            }
-
             // Full or partial sync request?
             if ($partial === true) {
                 $this->_logger->info(sprintf(
@@ -275,7 +273,9 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
             // Save.
             $this->_collections->save();
 
-            $this->_logger->info('All synckeys confirmed. Continuing with SYNC');
+            $this->_logger->info(sprintf(
+                '[%s] All synckeys confirmed. Continuing with SYNC',
+                $this->_procid));
         }
 
         // If this is >= 12.1, see if we want a looping SYNC.
@@ -296,7 +296,7 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
             if ($changes !== true && $changes !== false) {
                 switch ($changes) {
                 case Horde_ActiveSync_Collections::COLLECTION_ERR_STALE:
-                    $this->_logger->err(sprintf(
+                    $this->_logger->notice(sprintf(
                         '[%s] Changes in cache detected during looping SYNC exiting here.',
                         $this->_procid));
                     return true;
@@ -322,7 +322,9 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
             empty($changes) &&
             $this->_collections->canSendEmptyResponse()) {
 
-            $this->_logger->info('Sending an empty SYNC response.');
+            $this->_logger->info(sprintf(
+                '[%s] Sending an empty SYNC response.',
+                $this->_procid));
             $this->_collections->lastsyncendnormal = time();
             $this->_collections->save();
             return true;
@@ -348,7 +350,7 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
             try {
                 $this->_collections->initCollectionState($collection);
             } catch (Horde_ActiveSync_Exception_StateGone $e) {
-                $this->_logger->err(sprintf(
+                $this->_logger->notice(sprintf(
                     '[%s] SYNC terminating, state not found',
                     $this->_procid)
                 );
@@ -358,6 +360,9 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                 // iOS need this in order to catch missing state.
                 $this->_logger->err($e->getMessage());
                 $statusCode = self::STATUS_FOLDERSYNC_REQUIRED;
+            } catch (Horde_ActiveSync_Exception_StaleState $e) {
+                $this->_logger->notice($e->getMessage());
+                return false;
             } catch (Horde_ActiveSync_Exception $e) {
                 $this->_logger->err($e->getMessage());
                 return false;
@@ -371,11 +376,19 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
             // to be sure we don't screw up state. Any detected changes will be
             // ignored until the next cycle, utilizing the existing mechanism
             // for sending MOREITEMS when we will return the previously
-            // selected changes.
-            if (!empty($collection['importedchanges']) && empty($collection['getchanges'])) {
+            // selected changes. getchanges defaults to true if it is missing
+            // and the synckey != 0, defaults to true if it is present as an
+            // empty tag. If it is present, but '0' or not present but synckey
+            // is 0 then it defaults to false.
+            if (!isset($collection['getchanges']) && $collection['synckey'] != '0') {
+                $collection['getchanges'] = true;
+            }
+            if (!empty($collection['importedchanges']) && (empty($collection['getchanges']))) {
                 $forceChanges = true;
                 $collection['getchanges'] = true;
-                $this->_logger->notice('Force a GETCHANGES due to incoming changes.');
+                $this->_logger->notice(sprintf(
+                    '[%s] Force a GETCHANGES due to incoming changes.',
+                    $this->_procid));
             }
 
             if ($statusCode == self::STATUS_SUCCESS &&
@@ -397,11 +410,11 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                         $id);
                     $statusCode = self::STATUS_KEYMISM;
                 } catch (Horde_ActiveSync_Exception_StateGone $e) {
-                    $this->_logger->err(sprintf(
+                    $this->_logger->warn(sprintf(
                         '[%s] SYNCKEY not found. Reset required.', $this->_procid));
                         $statusCode = self::STATUS_KEYMISM;
                 } catch (Horde_ActiveSync_Exception_FolderGone $e) {
-                    $this->_logger->err(sprintf(
+                    $this->_logger->warn(sprintf(
                         '[%s] FOLDERSYNC required, collection gone.',
                         $this->_procid));
                     $statusCode = self::STATUS_FOLDERSYNC_REQUIRED;
@@ -422,13 +435,11 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                 !empty($collection['fetchids']) ||
                 $this->_collections->hasPingChangeFlag($id))) {
 
-                // Increment the loop detection counter.
-                $this->_collections->incrementLoopCounter($id, $collection['synckey']);
-
                 try {
                     $collection['newsynckey'] = $this->_state->getNewSyncKey($collection['synckey']);
                     $this->_logger->info(sprintf(
-                        'Old SYNCKEY: %s, New SYNCKEY: %s',
+                        '[%s] Old SYNCKEY: %s, New SYNCKEY: %s',
+                        $this->_procid,
                         $collection['synckey'],
                         $collection['newsynckey'])
                     );
@@ -467,8 +478,11 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                 $collection['mimesupport'] = 0;
             }
 
+            $ensure_sent = array();
             if ($statusCode == self::STATUS_SUCCESS) {
-                if (!empty($collection['clientids']) || !empty($collection['fetchids']) || !empty($collection['missing'])) {
+                if (!empty($collection['clientids']) || !empty($collection['fetchids'])
+                    || !empty($collection['missing']) || !empty($collection['importfailures'])) {
+
                     $this->_encoder->startTag(Horde_ActiveSync::SYNC_REPLIES);
 
                     // Output any errors from missing messages in REMOVE requests.
@@ -516,18 +530,25 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                         }
                     }
 
-                    // Output any SYNC_CHANGE failures
+                    // Output any SYNC_MODIFY failures
                     if (!empty($collection['importfailures'])) {
                         foreach ($collection['importfailures'] as $id => $reason) {
-                            $this->_encoder->startTag(Horde_ActiveSync::SYNC_CHANGE);
+                            $this->_encoder->startTag(Horde_ActiveSync::SYNC_MODIFY);
                             $this->_encoder->startTag(Horde_ActiveSync::SYNC_SERVERENTRYID);
-                            $tihs->_encoder->content($id);
+                            $this->_encoder->content($id);
                             $this->_encoder->endTag();
                             $this->_encoder->startTag(Horde_ActiveSync::SYNC_STATUS);
                             $this->_encoder->content($reason);
                             $this->_encoder->endTag();
                             $this->_encoder->endTag();
+                            // If we have a conflict, ensure we send the new server
+                            // data in the response, if possible. Some android
+                            // clients require this, or never accept the response.
+                            if ($reason == self::STATUS_CONFLICT) {
+                                $ensure_sent[] = $id;
+                            }
                         }
+
                     }
 
                     if (!empty($collection['fetchids'])) {
@@ -583,15 +604,17 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                     }
 
                     if (!empty($changecount)) {
-                        $exporter->setChanges($this->_collections->getCollectionChanges(), $collection);
+                        $exporter->setChanges($this->_collections->getCollectionChanges(false, $ensure_sent), $collection);
                         $this->_encoder->startTag(Horde_ActiveSync::SYNC_COMMANDS);
                         $cnt_collection = 0;
                         while ($cnt_collection < $collection['windowsize'] &&
                                $cnt_global < $this->_collections->getDefaultWindowSize() &&
                                $progress = $exporter->sendNextChange()) {
 
-                            ++$cnt_collection;
-                            ++$cnt_global;
+                            if ($progress === true) {
+                                ++$cnt_collection;
+                                ++$cnt_global;
+                            }
                         }
 
                         $this->_encoder->endTag();
@@ -629,7 +652,9 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
 
         if ($this->_device->version >= Horde_ActiveSync::VERSION_TWELVEONE) {
             if ($this->_collections->checkStaleRequest()) {
-                $this->_logger->info('Changes detected in sync_cache during wait interval, exiting without updating cache.');
+                $this->_logger->info(sprintf(
+                    '[%s] Changes detected in sync_cache during wait interval, exiting without updating cache.',
+                    $this->_procid));
                 return true;
             } else {
                 $this->_collections->lastsyncendnormal = time();
@@ -768,7 +793,9 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                     break;
 
                 case Horde_ActiveSync::SYNC_OPTIONS:
-                    $this->_parseSyncOptions($collection);
+                    if (!$this->_decoder->isEmptyElement($this->_decoder->getLastStartElement())) {
+                        $this->_parseSyncOptions($collection);
+                    }
                     break;
 
                 case Horde_ActiveSync::SYNC_COMMANDS:
@@ -784,13 +811,17 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                 return false;
             }
 
-            $this->_collections->addCollection($collection);
+            try {
+                $this->_collections->addCollection($collection);
+            } catch (Horde_ActiveSync_Exception $e) {
+                $this->_statusCode = self::STATUS_SERVERERROR;
+                $this->_handleError($collection);
+                return false;
+            }
+
             if (!empty($collection['importedchanges'])) {
                 $this->_collections->importedChanges = true;
             }
-            if ($collection['fetchids']) {
-                $this->_fetchids = true;
-            }
             if ($this->_collections->collectionExists($collection['id']) && !empty($collection['windowsize'])) {
                 $this->_collections->updateWindowSize($collection['id'], $collection['windowsize']);
             }
@@ -823,7 +854,7 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
         // This is a violation of the spec, and could lead to all kinds
         // of data integrity issues.
         if (empty($collection['synckey'])) {
-            $this->_logger->err(sprintf(
+            $this->_logger->warn(sprintf(
                 '[%s] Attempting a SYNC_COMMANDS, but device failed to send synckey. Ignoring.',
                 $this->_procid));
         }
@@ -838,10 +869,17 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
         try {
             $this->_collections->initCollectionState($collection);
         } catch (Horde_ActiveSync_Exception_StateGone $e) {
-            $this->_logger->err('State not found sending STATUS_KEYMISM');
+            $this->_logger->warn(sprintf(
+                '[%s] State not found sending STATUS_KEYMISM',
+                $this->_procid));
             $this->_statusCode = self::STATUS_KEYMISM;
             $this->_handleError($collection);
             return false;
+        } catch (Horde_ActiveSync_Exception_StaleState $e) {
+            $this->_logger->notice($e->getMessage());
+            $this->_statusCode = self::STATUS_SERVERERROR;
+            $this->_handleGlobalSyncError();
+            return false;
         } catch (Horde_ActiveSync_Exception $e) {
             $this->_logger->err($e->getMessage());
             $this->_statusCode = self::STATUS_SERVERERROR;
@@ -971,10 +1009,10 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                     if (isset($appdata)) {
                         $id = $importer->importMessageChange(
                             false, $appdata, $this->_device, $clientid, $collection['class']);
-                        if ($clientid && $id) {
+                        if ($clientid && $id && !is_array($id)) {
                             $collection['clientids'][$clientid] = $id;
                             $collection['importedchanges'] = true;
-                        } elseif (!$id) {
+                        } elseif (!$id || is_array($id)) {
                             $collection['clientids'][$clientid] = false;
                         }
                     }
@@ -1109,7 +1147,14 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                 }
             }
 
-            // @TODO Don't forget to add MAXITEMS when implementing RECIPIENTINFORMATION_CACHE.
+            if ($this->_decoder->getElementStartTag(Horde_ActiveSync::SYNC_MAXITEMS)) {
+                $options['maxitems'] = $this->_decoder->getElementContent();
+                if (!$this->_decoder->getElementEndTag()) {
+                    $this->_statusCode = self::STATUS_PROTERROR;
+                    $this->_handleError($collection);
+                    exit;
+                }
+            }
 
             // EAS 14.1
             if ($this->_device->version >= Horde_ActiveSync::VERSION_FOURTEENONE) {
