@@ -7,7 +7,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2010-2013 Horde LLC (http://www.horde.org)
+ * @copyright 2010-2014 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -20,7 +20,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2010-2013 Horde LLC (http://www.horde.org)
+ * @copyright 2010-2014 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  *
@@ -37,9 +37,6 @@
  *                                      looping sync.
  * @property integer lastsyncendnormal  Timestamp of the last looping sync that
  *                                      ended normally.
- * @property array synckeycounter       Track the number of times we see the
- *                                      same synckey for the same collection.
- *                                      Used for sync loop detection.
  */
 class Horde_ActiveSync_SyncCache
 {
@@ -78,6 +75,20 @@ class Horde_ActiveSync_SyncCache
      */
     protected $_logger;
 
+    /**
+     * Track dirty properties.
+     *
+     * @var array
+     */
+    protected $_dirty = array();
+
+    /**
+     * Process id for logging.
+     *
+     * @var integer
+     */
+    protected $_procid;
+
     /**
      * Constructor
      *
@@ -98,6 +109,7 @@ class Horde_ActiveSync_SyncCache
         $this->_user = $user;
         $this->_logger = $logger;
         $this->loadCacheFromStorage();
+        $this->_procid = getmypid();
     }
 
     public function __get($property)
@@ -115,6 +127,7 @@ class Horde_ActiveSync_SyncCache
             throw new InvalidArgumentException($property . ' is not a valid property');
         }
         $this->_data[$property] = $value;
+        $this->_dirty[$property] = true;
     }
 
     public function  __isset($property)
@@ -128,9 +141,8 @@ class Horde_ActiveSync_SyncCache
     protected function _isValidProperty($property)
     {
         return in_array($property, array(
-            'hbinterval', 'wait', 'hierarchy', 'confirmed_synckeys',
-            'lasthbsyncstarted', 'lastsyncendnormal', 'folders', 'pingheartbeat',
-            'synckeycounter'));
+            'hbinterval', 'wait', 'hierarchy', 'confirmed_synckeys', 'timestamp',
+            'lasthbsyncstarted', 'lastsyncendnormal', 'folders', 'pingheartbeat'));
     }
 
     /**
@@ -144,7 +156,11 @@ class Horde_ActiveSync_SyncCache
      */
     public function validateCache($hb_only = false)
     {
-        $cache = $this->_state->getSyncCache($this->_devid, $this->_user);
+        $cache = $this->_state->getSyncCache(
+            $this->_devid,
+            $this->_user,
+            array('lasthbsyncstarted', 'timestamp'));
+
         if ((!$hb_only && $cache['timestamp'] > $this->_data['timestamp']) ||
             ($cache['lasthbsyncstarted'] > $this->_data['lasthbsyncstarted'])) {
             return false;
@@ -159,6 +175,7 @@ class Horde_ActiveSync_SyncCache
     public function loadCacheFromStorage()
     {
         $this->_data = $this->_state->getSyncCache($this->_devid, $this->_user);
+        $this->_dirty = array();
     }
 
     /**
@@ -186,7 +203,7 @@ class Horde_ActiveSync_SyncCache
      */
     public function updateTimestamp()
     {
-        $this->_data['timestamp'] = time();
+        $this->timestamp = time();
     }
 
     /**
@@ -230,8 +247,11 @@ class Horde_ActiveSync_SyncCache
      */
     public function clearCollections()
     {
+        $this->_logger->info(sprintf(
+            '[%s] Clearing collections data from cache.',
+            $this->_procid));
         $this->_data['collections'] = array();
-        $this->_data['synckeycounter'] = array();
+        $this->_dirty['collections'] = true;
     }
 
     /**
@@ -257,6 +277,7 @@ class Horde_ActiveSync_SyncCache
             throw new InvalidArgumentException('Collection does not exist');
         }
         $this->_data['collections'][$id]['pingable'] = true;
+        $this->_markCollectionsDirty($id);
     }
 
     /**
@@ -264,12 +285,17 @@ class Horde_ActiveSync_SyncCache
      *
      * @param string $collectionid  The collection id.
      */
-    public function removePingableCollection($collectionid)
+    public function removePingableCollection($id)
     {
-         if (empty($this->_data['collections'][$collectionid])) {
-            throw new InvalidArgumentException('Collection does not exist');
+        if (empty($this->_data['collections'][$id])) {
+            $this->_logger->warn(sprintf(
+                '[%s] Collection %s was asked to be removed from PINGABLE but does not exist.',
+                $this->_procid,
+                $id));
+            return;
         }
-        $this->_data['collections'][$collectionid]['pingable'] = false;
+        $this->_data['collections'][$id]['pingable'] = false;
+        $this->_markCollectionsDirty($id);
     }
 
     /**
@@ -300,6 +326,7 @@ class Horde_ActiveSync_SyncCache
         }
 
         $this->_data['collections'][$id]['pingchange'] = true;
+        $this->_markCollectionsDirty($id);
     }
 
     /**
@@ -325,6 +352,7 @@ class Horde_ActiveSync_SyncCache
     public function resetPingChangeFlag($id)
     {
         $this->_data['collections'][$id]['pingchange'] = false;
+        $this->_markCollectionsDirty($id);
     }
 
     /**
@@ -334,7 +362,10 @@ class Horde_ActiveSync_SyncCache
     public function refreshCollections()
     {
         $syncCache = $this->_state->getSyncCache(
-            $this->_devid, $this->_user);
+            $this->_devid,
+            $this->_user,
+            array('collections')
+        );
         $cache_collections = $syncCache['collections'];
         foreach ($cache_collections as $id => $cache_collection) {
             if (!isset($cache_collection['lastsynckey'])) {
@@ -343,9 +374,11 @@ class Horde_ActiveSync_SyncCache
             $cache_collection['id'] = $id;
             $cache_collection['synckey'] = $cache_collection['lastsynckey'];
             $this->_data['collections'][$id] = $cache_collection;
+            $this->_markCollectionsDirty($id);
         }
         $this->_logger->info(sprintf(
-            '[%s] SyncCache collections refreshed.', getmypid()));
+            '[%s] SyncCache collections refreshed.',
+            $this->_procid));
     }
 
     /**
@@ -354,17 +387,21 @@ class Horde_ActiveSync_SyncCache
     public function save()
     {
         // Iterate over the collections and persist the last known synckey.
-        foreach ($this->_data['collections'] as &$collection) {
+        foreach ($this->_data['collections'] as $id => &$collection) {
             if (!empty($collection['synckey'])) {
                 $collection['lastsynckey'] = $collection['synckey'];
                 unset($collection['synckey']);
+                $this->_markCollectionsDirty($id);
             }
         }
-        $this->_data['timestamp'] = time();
+        $this->timestamp = time();
         $this->_state->saveSyncCache(
             $this->_data,
             $this->_devid,
-            $this->_user);
+            $this->_user,
+            $this->_dirty);
+
+        $this->_dirty = array();
     }
 
     /**
@@ -387,6 +424,7 @@ class Horde_ActiveSync_SyncCache
             'bodyprefs' => isset($collection['bodyprefs']) ? $collection['bodyprefs'] : null,
             'serverid' => isset($collection['serverid']) ? $collection['serverid'] : $collection['id']
         );
+        $this->_markCollectionsDirty($collection['id']);
     }
 
     /**
@@ -399,11 +437,16 @@ class Horde_ActiveSync_SyncCache
     public function removeCollection($id, $purge = true)
     {
         if (!$purge) {
+            $this->_logger->info(sprintf(
+                '[%s] Removing collection %s from SyncCache.',
+                $this->_procid, $id)
+            );
             unset($this->_data['collections'][$id]);
+            $this->_dirty['collections'] = true;
         } elseif (!empty($this->_data['collections'][$id])) {
             $this->_data['collections'][$id]['synckey'] = '0';
+            $this->_markCollectionsDirty($id);
         }
-        unset($this->_data['synckeycounter'][$id]);
     }
 
     /**
@@ -415,6 +458,7 @@ class Horde_ActiveSync_SyncCache
     public function updateWindowSize($id, $windowsize)
     {
         $this->_data['collections'][$id]['windowsize'] = $windowsize;
+        $this->_markCollectionsDirty($id);
     }
 
     /**
@@ -423,9 +467,14 @@ class Horde_ActiveSync_SyncCache
      */
     public function clearCollectionKeys()
     {
-        foreach ($this->_data['collections'] as &$c) {
+        $this->_logger->info(sprintf(
+            '[%s] Clearing all collection synckeys from the cache.',
+            $this->_procid)
+        );
+        foreach ($this->_data['collections'] as $id => &$c) {
             unset($c['synckey']);
         }
+        $this->_dirty['collections'] = true;
     }
 
     /**
@@ -436,6 +485,7 @@ class Horde_ActiveSync_SyncCache
     public function addConfirmedKey($key)
     {
         $this->_data['confirmed_synckeys'][$key] = true;
+        $this->_dirty['confirmed_synckeys'] = true;
     }
 
     /**
@@ -470,56 +520,72 @@ class Horde_ActiveSync_SyncCache
         if (!empty($collection['id'])) {
             if ($options['newsynckey']) {
                 $this->_data['collections'][$collection['id']]['synckey'] = $collection['newsynckey'];
+                $this->_markCollectionsDirty($collection['id']);
             } elseif (isset($collection['synckey'])) {
                 $this->_data['collections'][$collection['id']]['synckey'] = $collection['synckey'];
+                $this->_markCollectionsDirty($collection['id']);
             }
             if (isset($collection['class'])) {
                 $this->_data['collections'][$collection['id']]['class'] = $collection['class'];
+                $this->_markCollectionsDirty($collection['id']);
             }
             if (isset($collection['windowsize'])) {
                 $this->_data['collections'][$collection['id']]['windowsize'] = $collection['windowsize'];
+                $this->_markCollectionsDirty($collection['id']);
             }
             if (isset($collection['deletesasmoves'])) {
                 $this->_data['collections'][$collection['id']]['deletesasmoves'] = $collection['deletesasmoves'];
+                $this->_markCollectionsDirty($collection['id']);
             }
             if (isset($collection['filtertype'])) {
                 $this->_data['collections'][$collection['id']]['filtertype'] = $collection['filtertype'];
+                $this->_markCollectionsDirty($collection['id']);
             }
             if (isset($collection['truncation'])) {
                 $this->_data['collections'][$collection['id']]['truncation'] = $collection['truncation'];
+                $this->_markCollectionsDirty($collection['id']);
             }
             if (isset($collection['rtftruncation'])) {
                 $this->_data['collections'][$collection['id']]['rtftruncation'] = $collection['rtftruncation'];
+                $this->_markCollectionsDirty($collection['id']);
             }
             if (isset($collection['mimesupport'])) {
                 $this->_data['collections'][$collection['id']]['mimesupport'] = $collection['mimesupport'];
+                $this->_markCollectionsDirty($collection['id']);
             }
             if (isset($collection['mimetruncation'])) {
                 $this->_data['collections'][$collection['id']]['mimetruncation'] = $collection['mimetruncation'];
+                $this->_markCollectionsDirty($collection['id']);
             }
             if (isset($collection['conflict'])) {
                 $this->_data['collections'][$collection['id']]['conflict'] = $collection['conflict'];
+                $this->_markCollectionsDirty($collection['id']);
             }
             if (isset($collection['bodyprefs'])) {
                 $this->_data['collections'][$collection['id']]['bodyprefs'] = $collection['bodyprefs'];
+                $this->_markCollectionsDirty($collection['id']);
             }
             if (isset($collection['pingable'])) {
                 $this->_data['collections'][$collection['id']]['pingable'] = $collection['pingable'];
+                $this->_markCollectionsDirty($collection['id']);
             }
             if (isset($collection['serverid'])) {
                 $this->_data['collections'][$collection['id']]['serverid'] = $collection['serverid'];
+                $this->_markCollectionsDirty($collection['id']);
             }
             if ($options['unsetChanges']) {
                 unset($this->_data['collections'][$collection['id']]['getchanges']);
+                $this->_markCollectionsDirty($collection['id']);
             }
             if ($options['unsetPingChangeFlag']) {
                 unset($this->_data['collections'][$collection['id']]['pingchange']);
+                $this->_markCollectionsDirty($collection['id']);
             }
-
         } else {
             $this->_logger->info(sprintf(
-                'Collection without id found: %s',
-                print_r($collection, true))
+                '[%s] Collection without id found: %s',
+                $this->_procid,
+                serialize($collection))
             );
         }
     }
@@ -535,24 +601,31 @@ class Horde_ActiveSync_SyncCache
         foreach ($collections as $key => $values) {
             if (!isset($values['class']) && isset($this->_data['folders'][$values['id']]['class'])) {
                 $collections[$key]['class'] = $this->_data['folders'][$values['id']]['class'];
+                $this->_markCollectionsDirty($key);
             }
             if (!isset($values['filtertype']) && isset($this->_data['collections'][$values['id']]['filtertype'])) {
                 $collections[$key]['filtertype'] = $this->_data['collections'][$values['id']]['filtertype'];
+                $this->_markCollectionsDirty($key);
             }
             if (!isset($values['mimesupport']) && isset($this->_data['collections'][$values['id']]['mimesupport'])) {
                 $collections[$key]['mimesupport'] = $this->_data['collections'][$values['id']]['mimesupport'];
+                $this->_markCollectionsDirty($key);
             }
             if (empty($values['bodyprefs']) && isset($this->_data['collections'][$values['id']]['bodyprefs'])) {
                 $collections[$key]['bodyprefs'] = $this->_data['collections'][$values['id']]['bodyprefs'];
+                $this->_markCollectionsDirty($key);
             }
             if (empty($values['truncation']) && isset($this->_data['collections'][$values['id']]['truncation'])) {
                 $collections[$key]['truncation'] = $this->_data['collections'][$values['id']]['truncation'];
+                $this->_markCollectionsDirty($key);
             }
             if (empty($values['mimetruncation']) && isset($this->_data['collections'][$values['id']]['mimetruncation'])) {
                 $collections[$key]['mimetruncation'] = $this->_data['collections'][$values['id']]['mimetruncation'];
+                $this->_markCollectionsDirty($key);
             }
             if (empty($values['serverid']) && isset($this->_data['collections'][$values['id']]['serverid'])) {
                 $collections[$key]['serverid'] = $this->_data['collections'][$values['id']]['serverid'];
+                $this->_markCollectionsDirty($key);
             }
 
             if (!isset($values['windowsize'])) {
@@ -560,6 +633,7 @@ class Horde_ActiveSync_SyncCache
                     isset($this->_data['collections'][$values['id']]['windowsize'])
                         ? $this->_data['collections'][$values['id']]['windowsize']
                         : 100;
+                $this->_markCollectionsDirty($key);
             }
 
             // in case the maxitems (windowsize) is above 512 or 0 it should be
@@ -568,6 +642,7 @@ class Horde_ActiveSync_SyncCache
                 $collections[$key]['windowsize'] == 0) {
 
                 $collections[$key]['windowsize'] = self::MAX_WINDOW_SIZE;
+                $this->_markCollectionsDirty($key);
             }
 
             if (isset($values['synckey']) &&
@@ -576,10 +651,12 @@ class Horde_ActiveSync_SyncCache
                 $this->_data['collections'][$values['id']]['synckey'] != '0') {
 
                 unset($this->_data['collections'][$values['id']]['synckey']);
+                $this->_markCollectionsDirty($key);
             }
 
             if (!isset($values['pingable']) && isset($this->_data['collections'][$values['id']]['pingable'])) {
                 $collections[$key]['pingable'] = $this->_data['collections'][$values['id']]['pingable'];
+                $this->_markCollectionsDirty($key);
             }
         }
     }
@@ -601,6 +678,7 @@ class Horde_ActiveSync_SyncCache
     public function clearFolders()
     {
         $this->_data['folders'] = array();
+        $this->_dirty['folders'] = true;
     }
 
     /**
@@ -631,6 +709,7 @@ class Horde_ActiveSync_SyncCache
             $this->_data['folders'][$folder->serverid] = array('class' => 'Email');
         }
         $this->_data['folders'][$folder->serverid]['serverid'] = $folder->_serverid;
+        $this->_dirty['folders'] = true;
     }
 
     /**
@@ -642,6 +721,8 @@ class Horde_ActiveSync_SyncCache
     {
         unset($this->_data['folders'][$folder]);
         unset($this->_data['collections'][$folder]);
+        $this->_dirty['folders'] = true;
+        $this->_markCollectionsDirty($folder);
     }
 
     /**
@@ -663,6 +744,23 @@ class Horde_ActiveSync_SyncCache
     {
         $this->_state->deleteSyncCache($this->_devid, $this->_user);
         $this->_data = array();
+        $this->_dirty = array();
+    }
+
+    /**
+     * Mark specific collection as dirty, but only if the entire collection
+     * data is not already marked dirty.
+     *
+     * @param boolean $id  The collection to mark dirty.
+     */
+    protected function _markCollectionsDirty($id)
+    {
+        if (isset($this->_dirty['collections']) && is_array($this->_dirty['collections'])) {
+            $this->_dirty['collections'][$id] = true;
+        } elseif (!isset($this->_dirty['collections']) || $this->_dirty['collections'] !== true) {
+            $this->_dirty['collections'] = array();
+            $this->_markCollectionsDirty($id);
+        }
     }
 
 }
\ No newline at end of file
