@@ -18,7 +18,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2009-2013 Horde LLC (http://www.horde.org)
+ * @copyright 2009-2014 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -30,7 +30,7 @@
  *            Version 2, the distribution of the Horde_ActiveSync module in or
  *            to the United States of America is excluded from the scope of this
  *            license.
- * @copyright 2009-2013 Horde LLC (http://www.horde.org)
+ * @copyright 2009-2014 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -63,11 +63,19 @@ class Horde_ActiveSync_Wbxml_Decoder extends Horde_ActiveSync_Wbxml
      *
      * @var boolean
      */
-    private $_isWbxml;
+    protected $_isWbxml;
 
-    private $_attrcp = 0;
-    private $_ungetbuffer;
-    private $_readHeader = false;
+    protected $_attrcp = 0;
+    protected $_ungetbuffer;
+    protected $_readHeader = false;
+
+    /**
+     * Cache the last successfully fetched start tag array. Used to be able
+     * to easily detected emtpy nodes after the element was already fetched.
+     *
+     * @var array
+     */
+    protected $_lastStartElement;
 
     /**
      * Start reading the wbxml stream, pulling off the initial header and
@@ -79,8 +87,8 @@ class Horde_ActiveSync_Wbxml_Decoder extends Horde_ActiveSync_Wbxml
         $this->_readVersion();
         if ($this->version != self::WBXML_VERSION) {
             // Not Wbxml - save the byte we already read.
-            $this->_buffer = fopen('php://temp/maxmemory:2097152', 'r+');
-            fwrite($this->_buffer, chr($this->version));
+            $this->_getTempStream();
+            $this->_buffer->add(chr($this->version));
             $this->_isWbxml = false;
             return;
         } else {
@@ -120,22 +128,17 @@ class Horde_ActiveSync_Wbxml_Decoder extends Horde_ActiveSync_Wbxml
     public function getFullInputStream()
     {
         // Ensure the buffer was created
-        if (!isset($this->_buffer)) {
-            $this->_buffer = fopen('php://temp/maxmemory:2097152', 'r+');
-        }
-        while (!feof($this->_stream)) {
-            fwrite($this->_buffer, fread($this->_stream, 8192));
-        }
-        rewind($this->_buffer);
-
-        return $this->_buffer;
+        $this->_getTempStream();
+        $this->_buffer->add($this->_stream);
+        $this->_buffer->rewind();
+        return $this->_buffer->stream;
     }
 
     /**
      * Returns either start, content or end, and auto-concatenates successive
      * content.
      *
-     * @return mixed  The element requested or false on failure.
+     * @return mixed  The element false on failure.
      */
     public function getElement()
     {
@@ -164,6 +167,38 @@ class Horde_ActiveSync_Wbxml_Decoder extends Horde_ActiveSync_Wbxml
         return false;
     }
 
+    /**
+     * Returns whether or not the passed in element array represents an
+     * empty tag.
+     *
+     * @param  array $el  The element array.
+     *
+     * @return boolean  True if $el is an empty start tag, otherwise false.
+     */
+    public function isEmptyElement($el)
+    {
+        if (!is_array($el)) {
+            return false;
+        }
+        switch ($el[self::EN_TYPE]) {
+        case self::EN_TYPE_STARTTAG:
+            return !($el[self::EN_FLAGS] & self::EN_FLAGS_CONTENT);
+        default:
+            // Not applicable.
+            return false;
+        }
+    }
+
+    /**
+     * Returns the last element array fetched using getElementStartTag()
+     *
+     * @return array|boolean  The element array, or false if none available.
+     */
+    public function getLastStartElement()
+    {
+        return $this->_lastStartElement;
+    }
+
     /**
      * Peek at the next element in the stream.
      *
@@ -191,8 +226,10 @@ class Horde_ActiveSync_Wbxml_Decoder extends Horde_ActiveSync_Wbxml
         if ($element[self::EN_TYPE] == self::EN_TYPE_STARTTAG &&
             $element[self::EN_TAG] == $tag) {
 
+            $this->_lastStartElement = $element;
             return $element;
         } else {
+            $this->_lastStartElement = false;
             $this->_ungetElement($element);
         }
 
@@ -263,7 +300,7 @@ class Horde_ActiveSync_Wbxml_Decoder extends Horde_ActiveSync_Wbxml
      *
      * @return void
      */
-    private function _logToken($el)
+    protected function _logToken($el)
     {
         switch ($el[self::EN_TYPE]) {
         case self::EN_TYPE_STARTTAG:
@@ -295,11 +332,22 @@ class Horde_ActiveSync_Wbxml_Decoder extends Horde_ActiveSync_Wbxml
             break;
         case self::EN_TYPE_CONTENT:
             $spaces = str_repeat(' ', count($this->_logStack) + 1);
-            $this->_logger->debug(sprintf(
-                '[%s] I %s %s',
-                $this->_procid,
-                $spaces,
-                $el[self::EN_CONTENT]));
+            if ($this->_logLevel == self::LOG_PROTOCOL &&
+                ($l = Horde_String::length($el[self::EN_CONTENT])) > self::LOG_MAXCONTENT) {
+                $this->_logger->debug(sprintf(
+                    '[%s] I %s %s',
+                    $this->_procid,
+                    $spaces,
+                    sprintf('[%d bytes of content]', $l)
+                  ));
+            } else {
+                $this->_logger->debug(sprintf(
+                    '[%s] I %s %s',
+                    $this->_procid,
+                    $spaces,
+                    $el[self::EN_CONTENT])
+                );
+            }
             break;
         }
     }
@@ -309,7 +357,7 @@ class Horde_ActiveSync_Wbxml_Decoder extends Horde_ActiveSync_Wbxml
      *
      * @return array  The element array.
      */
-   private function _getToken() {
+   protected function _getToken() {
 
         // Get the data from the input stream
         $element = array();
@@ -433,7 +481,7 @@ class Horde_ActiveSync_Wbxml_Decoder extends Horde_ActiveSync_Wbxml
      * Read the Wbxml version header byte, and buffer the input incase we
      * need the full stream later.
      */
-    private function _readVersion()
+    protected function _readVersion()
     {
         $b = $this->_getByte();
         if ($b != NULL) {
@@ -446,7 +494,7 @@ class Horde_ActiveSync_Wbxml_Decoder extends Horde_ActiveSync_Wbxml
      *
      * @return mixed  The value of the element's attributes.
      */
-    private function _getAttributes()
+    protected function _getAttributes()
     {
         $attributes = array();
         $attr = '';
@@ -542,7 +590,7 @@ class Horde_ActiveSync_Wbxml_Decoder extends Horde_ActiveSync_Wbxml
      *
      * @return array  The attribute hash
      */
-    private function _splitAttribute($attr)
+    protected function _splitAttribute($attr)
     {
         $attributes = array();
         $pos = strpos($attr,chr(61)); // equals sign
@@ -560,7 +608,7 @@ class Horde_ActiveSync_Wbxml_Decoder extends Horde_ActiveSync_Wbxml
      *
      * @return string  The string
      */
-    private function _getTermStr()
+    protected function _getTermStr()
     {
         $str = '';
         while(1) {
@@ -583,7 +631,7 @@ class Horde_ActiveSync_Wbxml_Decoder extends Horde_ActiveSync_Wbxml
      *
      * @return string  A string of bytes representing the opaque value.
      */
-    private function _getOpaque($len)
+    protected function _getOpaque($len)
     {
         // See http://php.net/fread for why we can't simply use a single fread()
         // here. Bottom line, for buffered network streams it may be possible
@@ -593,9 +641,9 @@ class Horde_ActiveSync_Wbxml_Decoder extends Horde_ActiveSync_Wbxml
         while (1) {
             $l = (($len - strlen($d)) > 8192) ? 8192 : ($len - strlen($d));
             if ($l > 0) {
-                $data = fread($this->_stream, $l);
+                $data = $this->_stream->substring(0, $l);
                 // Stream ends prematurely on instable connections and big mails
-                if ($data === false || feof($this->_stream)) {
+                if ($data === false || $this->_stream->eof()) {
                     throw new Horde_ActiveSync_Exception(sprintf(
                         'Connection unavailable while trying to read %d bytes from stream. Aborting after %d bytes read.',
                         $len,
@@ -617,9 +665,9 @@ class Horde_ActiveSync_Wbxml_Decoder extends Horde_ActiveSync_Wbxml
      *
      * @return string  The single byte.
      */
-    private function _getByte()
+    protected function _getByte()
     {
-        $ch = fread($this->_stream, 1);
+        $ch = $this->_stream->getChar();
         if (strlen($ch) > 0) {
             $ch = ord($ch);
             return $ch;
@@ -633,7 +681,7 @@ class Horde_ActiveSync_Wbxml_Decoder extends Horde_ActiveSync_Wbxml
      *
      * @return integer
      */
-    private function _getMBUInt()
+    protected function _getMBUInt()
     {
         $uint = 0;
         while (1) {
@@ -654,12 +702,12 @@ class Horde_ActiveSync_Wbxml_Decoder extends Horde_ActiveSync_Wbxml
      *
      * @return string  The string table.
      */
-    private function _getStringTable()
+    protected function _getStringTable()
     {
         $stringtable = '';
         $length = $this->_getMBUInt();
         if ($length > 0) {
-            $stringtable = fread($this->_stream, $length);
+            $stringtable = $this->_stream->substring(0, $length);
         }
 
         return $stringtable;
@@ -676,7 +724,7 @@ class Horde_ActiveSync_Wbxml_Decoder extends Horde_ActiveSync_Wbxml
      *
      * @return string
      */
-    private function _getStringTableEntry($id)
+    protected function _getStringTableEntry($id)
     {
         throw new Horde_ActiveSync_Exception('Not implemented');
     }
@@ -689,7 +737,7 @@ class Horde_ActiveSync_Wbxml_Decoder extends Horde_ActiveSync_Wbxml
      *
      * @return mixed  The mapped value.
      */
-    private function _getMapping($cp, $id)
+    protected function _getMapping($cp, $id)
     {
         if (!isset($this->_dtd['codes'][$cp]) || !isset($this->_dtd['codes'][$cp][$id])) {
             return false;
@@ -702,4 +750,18 @@ class Horde_ActiveSync_Wbxml_Decoder extends Horde_ActiveSync_Wbxml
         }
     }
 
-}
\ No newline at end of file
+    /**
+     * Return the temporary buffer stream.
+     *
+     * @return stream
+     */
+    protected function _getTempStream()
+    {
+        if (!isset($this->_buffer)) {
+            $this->_buffer = new Horde_Stream_Temp();
+        }
+
+        return $this->_buffer;
+    }
+
+}
