@@ -8,7 +8,7 @@
  */
 class Horde_ActiveSync_ServerTest extends Horde_Test_Case
 {
-    static $_server;
+    static protected $_server;
     public function setup()
     {
         $driver = $this->getMockSkipConstructor('Horde_ActiveSync_Driver_Base');
@@ -39,4 +39,4 @@ class Horde_ActiveSync_ServerTest extends Horde_Test_Case
         $this->assertEquals('Sync,SendMail,SmartForward,SmartReply,GetAttachment,GetHierarchy,CreateCollection,DeleteCollection,MoveCollection,FolderSync,FolderCreate,FolderDelete,FolderUpdate,MoveItems,GetItemEstimate,MeetingResponse,ResolveRecipients,ValidateCert,Provision,Search,Ping', self::$_server->getSupportedCommands());
     }
 
-}
\ No newline at end of file
+}
