@@ -31,16 +31,78 @@ class Horde_ActiveSync_StateTest_Base extends Horde_Test_Case
         $deviceInfo->userAgent = 'Horde Tests';
         $deviceInfo->id = 'dev123';
         $deviceInfo->user = 'mike';
-        $deviceInfo->policykey = 0;
+        $deviceInfo->policykey = 456;
         $deviceInfo->supported = array();
-
-        self::$state->setDeviceInfo($deviceInfo);
+        $deviceInfo->save();
         $this->assertEquals(true, (boolean)self::$state->deviceExists('dev123', 'mike'));
 
         $di = self::$state->loadDeviceInfo('dev123', 'mike');
         $this->assertEquals($deviceInfo, $di);
     }
 
+    /**
+     * @return [type] [description]
+     */
+    protected function _testListDevices()
+    {
+        $devices = self::$state->listDevices();
+        $this->assertCount(1, $devices);
+
+        $deviceInfo = new Horde_ActiveSync_Device(self::$state);
+        $deviceInfo->rwstatus = 0;
+        $deviceInfo->deviceType = 'Test Device';
+        $deviceInfo->userAgent = 'Horde Tests';
+        $deviceInfo->id = 'dev123';
+        $deviceInfo->user = 'ashley';
+        $deviceInfo->policykey = 123;
+        $deviceInfo->supported = array();
+        $deviceInfo->save();
+
+        $devices = self::$state->listDevices();
+        $this->assertCount(2, $devices);
+    }
+
+    protected function _testPolicyKeys()
+    {
+        $device = self::$state->loadDeviceInfo('dev123', 'mike');
+        $this->assertEquals(456, $device->policykey);
+        self::$state->setPolicyKey('dev123', 789);
+
+        // Make sure it took without affected other data
+        // (need to load a different device first, to clear
+        // the local copy of the data)
+        $device = self::$state->loadDeviceInfo('dev123', 'ashley');
+        $this->assertEquals(123, $device->policykey);
+        $device = self::$state->loadDeviceInfo('dev123', 'mike');
+        $this->assertEquals(789, $device->policykey);
+
+        self::$state->resetAllPolicyKeys();
+        $device = self::$state->loadDeviceInfo('dev123', 'ashley');
+        $this->assertEquals(0, $device->policykey);
+        $device = self::$state->loadDeviceInfo('dev123', 'mike');
+        $this->assertEquals(0, $device->policykey);
+    }
+
+    protected function _testDuplicatePIMAddition()
+    {
+        // @TODO. Need to implement getChanges/saveState tests.
+    }
+
+    /**
+     * Not much testing here yet, just run through a save and check for
+     * fatals.
+     *
+     */
+    protected function _loadStateTest()
+    {
+        $collection = array(
+            'folderid' => '@Contacts@',
+            'class' => Horde_ActiveSync::CLASS_CONTACTS);
+        self::$state->loadState($collection, 0, Horde_ActiveSync::REQUEST_TYPE_SYNC, 'abcdef');
+        self::$state->setNewSyncKey('{51941e99-0b9c-41f8-b678-1532c0a8015f}1');
+        self::$state->save();
+    }
+
     protected function _testCacheInitialState()
     {
         $cache = new Horde_ActiveSync_SyncCache(self::$state, 'dev123', 'mike', self::$logger->getLogger());
@@ -92,6 +154,14 @@ class Horde_ActiveSync_StateTest_Base extends Horde_Test_Case
         $cache->save();
     }
 
+    protected function _testCacheDataRestrictFields()
+    {
+        $cache_data = self::$state->getSyncCache('dev123', 'mike', array('folders'));
+        $this->assertCount(1, $cache_data);
+        list($key, $value) = each($cache_data);
+        $this->assertEquals('folders', $key);
+    }
+
     protected function _testCacheFoldersPersistence()
     {
         $cache = new Horde_ActiveSync_SyncCache(self::$state, 'dev123', 'mike', self::$logger->getLogger());
@@ -515,4 +585,4 @@ class Horde_ActiveSync_StateTest_Base extends Horde_Test_Case
         return $collections;
     }
 
-}
\ No newline at end of file
+}
